

-define(SERVER_CONFIG_H, true).

-type primary_key(X) :: X.
-type disk_path() :: binary().
-type network_port() :: 1..65536.
-type network_addr() :: binary().
-type network_version() :: ipv4 | ipv6.
-type network_transport() :: tcp | udp.
-type network_mask() :: 1..32.
-type interface_mode() :: static | dynamic.
-type hostport() :: binary().
-type host_with_port() :: binary().
-type listen_spec() :: network_port() | host_with_port().
-type aliaser_url() :: binary().
-type bitrate() :: speed().
-type hexbinary() :: binary().
-type cache_name() :: binary().
-type agent_url() :: binary().
-type url_prefix() :: false | binary().
-type url() :: binary().
-type web_mount_point() :: binary().
-type loglevel() :: none | debug | info | notice | warning | error | alert | critical.
-type sort_index() :: non_neg_integer().
-type auth_token() :: binary().
-type server_meta() :: binary().
-type iso8601() :: seconds(). % 2016-07-08T09:04:46Z
-type decibels() :: integer().
-type iso3166() :: binary(). % country code

-type chassis_version() :: binary().
-type server_version() :: binary().
-type previous_server_state() :: shutdown | killed | unknown | none | error_reading | unconfigured.
-type config_version() :: [non_neg_integer()].
-type agent_version() :: binary().

-type instance_id() :: uuid().
-type server_id() :: uuid().

-type license_type() :: undefined | online | usb | network_usb.

-type session_id() :: binary().
-type session_token() :: binary().
-type country() :: binary().
-type session_user_id() :: binary().
-type temp_session_id() :: session_id().
-type protocol() :: atom(). % hls | rtmp | rtsp ...
-type play_protocol() :: protocol().
-type source_protocol() :: protocol().
-type publish_protocol() :: protocol().
-type push_protocol() :: protocol().

% -type play_protocol() :: atom(). % hls | rtmp | rtsp ...
% -type source_protocol() :: atom().
% -type publish_protocol() :: rtmp | rtmps | rtsp | rtsps | webrtc | srt | m4s | tshttp.
% -type push_protocol() :: hls | m4f | m4fs | tshttp | m4s | m4ss | rtmp | rtmps | udp | udp1 | udp2 | decklink | aja | v4l.
-type user_agent() :: binary().

-type session_identity() :: any().
-type session_status() :: establishing | running | stalling | finished.


-type listen_protocol() :: http | admin_http | https | admin_https | mdns | rtmp | rtmps | rtsp | rtsps | srt.

-type file_access_method() :: file | http_file | http_file2 | vod_mbr_dir.

-type motion_id() :: uuid().
-type jpeg_base64() :: binary().

-type webrtc_prefer_codec() :: h264 | vp8 | vp9.
-type webrtc_output_audio() :: add_aac | aac | keep.

-type agent_id() :: binary().
-type agent_capability() :: tcp2 | keepalive | restart_badauth.
-type agent_reqid() :: binary().

-type dvr_blob_path() :: binary().
-type segment_path() :: binary().

-type drm_resource_id() :: binary().
-type drm_content_type() :: vod | live.

-type auth_url() :: boolean() | mocked | binary() | any().

-type client_count() :: integer().

-type binary_int_list() :: binary().

-type logo_path() :: binary().
-type runtime(A) :: A.

-type drm_system() :: widevine | fairplay | playready.











% $$\      $$\ $$$$$$$\  $$$$$$$$\  $$$$$$\       $$$$$$$$\  $$$$$$\  
% $$$\    $$$ |$$  __$$\ $$  _____|$$  __$$\      \__$$  __|$$  __$$\ 
% $$$$\  $$$$ |$$ |  $$ |$$ |      $$ /  \__|        $$ |   $$ /  \__|
% $$\$$\$$ $$ |$$$$$$$  |$$$$$\    $$ |$$$$\ $$$$$$\ $$ |   \$$$$$$\  
% $$ \$$$  $$ |$$  ____/ $$  __|   $$ |\_$$ |\______|$$ |    \____$$\ 
% $$ |\$  /$$ |$$ |      $$ |      $$ |  $$ |        $$ |   $$\   $$ |
% $$ | \_/ $$ |$$ |      $$$$$$$$\ \$$$$$$  |        $$ |   \$$$$$$  |
% \__|     \__|\__|      \________| \______/         \__|    \______/ 






-type mpegts_cc_error_action() :: none | log | repeat | drop.



-type psi_name() :: psi | pat | pmt | nit | sdt | cat.
-type psi_version() :: 0..31.
-type program_id() :: 1..65535.
-type ts_pid() :: 0..8191.
-type vbi_service() :: ttxt.
-type vbi_line() :: 6..23 | 318..335.



-type ts_service_type() :: non_neg_integer() |
  digital_tv |
  digital_radio |
  teletext |
  nvod_reference |
  nvod_shifted |
  mosaic |
  fm_radio |
  dvb_srm |
  digital_radio_aac |
  mosaic_avc |
  data |
  common_interface |
  rcs_map |
  rcs_fls |
  dvb_mhp |
  digital_tv_mpeg2_hd |
  digital_tv_avc_sd |
  nvod_shifted_avc_sd |
  nvod_reference_avc_sd |
  digital_tv_avc_hd |
  nvod_shifted_avc_sd |
  nvod_reference_avc_sd |
  digital_tv_avc_stereo_hd |
  nvod_shifted_avc_stereo_hd |
  nvod_reference_avc_stereo_hd |
  digital_tv_hevc |
  digital_tv_hevc_uhd.










-record(raw_ts_descriptor, {
  tag :: integer(),
  hex :: hexbinary()
}).

-type transponder_name() :: binary().

-type transponder_timeout() :: service_type|media_info.

-record(transponder_pid, {
  pid :: primary_key(ts_pid()),
  position = undefined :: sort_index(),
  content :: system | video | audio | text | application,
  track :: pmt | sdt | non_neg_integer(),
  pcr = false :: boolean(),
  bitrate = undefined :: bitrate()
}).


-record(transponder_program, {
  program_id :: primary_key(program_id()),
  position = undefined :: sort_index(),
  source = undefined :: stream_name(),
  lcn = undefined :: non_neg_integer(),
  service_type = undefined :: ts_service_type(),
  vb = undefined :: bitrate(),
  title = undefined :: binary(),
  pids = #{} :: #{ts_pid() => #transponder_pid{}},
  eit_title = undefined :: binary()
}).


-record(eit_intervals, {
  actual = undefined :: non_neg_integer(),
  other = undefined :: non_neg_integer()
}).


-record(transponder_eit, {
  max_bitrate = undefined :: bitrate(),
  xmltv_url = undefined :: binary(),
  keep_epg = undefined :: seconds(),
  intervals_pf = #{} :: #eit_intervals{},
  intervals_schedule = #{} :: #eit_intervals{}
}).


-record(transponder_push, {
  url :: binary(),
  multicast_loop = true :: boolean(),
  depth_limit = undefined :: binary_int(),
  pkt_limit = undefined :: binary_int()
}).


-record(transponder_time_offset, {
  country :: binary(),
  region :: non_neg_integer(),
  time_of_change = undefined :: iso8601(),
  local_time_offset = undefined :: binary(),
  next_time_offset = undefined :: binary()
}).


-record(transponder_config, {
  name :: primary_key(transponder_name()),
  position = undefined :: sort_index(),
  pushes = #{} :: #{binary() => #transponder_push{}},
  bitrate = undefined :: bitrate(),
  provider = undefined :: binary(),
  network_name = undefined :: binary(),
  ts_stream_id = undefined :: non_neg_integer(),
  network_id = undefined :: non_neg_integer(),
  original_network_id = undefined :: non_neg_integer(),
  raw_ts_descriptors = [] :: [#raw_ts_descriptor{}],
  timeouts = #{} :: #{transponder_timeout() => non_neg_integer()},
  programs = #{} :: #{program_id() => #transponder_program{}},
  time_offsets = [] :: [#transponder_time_offset{}],
  versions = #{} :: #{psi_name() => psi_version()},
  other = [] :: [transponder_name()],
  eit = undefined :: #transponder_eit{}
}).









% $$$$$$$\  $$\    $$\ $$$$$$$\  
% $$  __$$\ $$ |   $$ |$$  __$$\ 
% $$ |  $$ |$$ |   $$ |$$ |  $$ |
% $$ |  $$ |\$$\  $$  |$$$$$$$  |
% $$ |  $$ | \$$\$$  / $$  __$$< 
% $$ |  $$ |  \$$$  /  $$ |  $$ |
% $$$$$$$  |   \$  /   $$ |  $$ |
% \_______/     \_/    \__|  \__|





-type dvr_name() :: binary().
-type dvr_url() :: disk_path().
-type dvr_mount_path() :: disk_path().


-type dvr_schedule_range() :: [integer()].


-record(record_input_spec, {
  root = undefined :: dvr_url(),
  disk_space = undefined :: bytes(),
  dvr_limit = undefined :: seconds()
}).

-record(stream_dvr_spec, {
  root = undefined :: dvr_url(),
  reference = undefined :: dvr_name(),
  disk_space = undefined :: bytes(),
  disk_limit = undefined :: percents(),
  dvr_limit = undefined :: seconds(),
  dvr_replicate = undefined :: boolean(),
  replication_speed = undefined :: speed(),
  replication_port = undefined :: network_port(),
  no_index = undefined :: boolean(),
  schedule = undefined :: [dvr_schedule_range()],
  dvr_offline = undefined :: boolean(),
  locked_write = undefined :: boolean(),
  copy = undefined :: dvr_url()
}).


-type raid_disk_mode() :: normal | migrate | readonly | rescue | abandon | keep.

-record(raid_disk_stats, {
  disk = undefined :: dvr_mount_path(),
  path = undefined :: dvr_url(),
  status = undefined :: mounted | not_mounted,
  mode = undefined :: raid_disk_mode(),
  blobs_count = undefined :: non_neg_integer(),
  size = undefined :: bytes(),
  usage = undefined :: percents(),
  io_usage = undefined :: percents(),
  migration_speed = undefined :: speed(),
  migration_eta = undefined :: utc(),
  migration_updated = undefined :: utc()
}).


-record(raid_disk_config, {
  position = undefined :: sort_index(),
  mode = undefined :: raid_disk_mode(),
  stats = undefined :: #raid_disk_stats{}
}).


-record(dvr_config, {
  name :: primary_key(dvr_name()),
  position = undefined :: sort_index(),
  root :: dvr_url(),
  disk_space = undefined :: bytes(),
  disk_limit = undefined :: percents(),
  dvr_limit = undefined :: seconds(),
  dvr_replicate = undefined :: boolean(),
  replication_speed = undefined :: speed(),
  replication_port = undefined :: network_port(),
  no_index = undefined :: boolean(),
  schedule = undefined :: [dvr_schedule_range()],
  dvr_offline = undefined :: boolean(),
  copy = undefined :: dvr_url(),

  raid = undefined :: 0,
  check_mount = undefined :: boolean(),
  active = undefined :: non_neg_integer(),
  index = undefined :: dvr_mount_path(),
  disks = undefined :: #{dvr_mount_path() => #raid_disk_config{}}
}).


-type dvr_range() :: #{from => seconds(), duration => seconds()}.
-type dvr_motion_event() :: #{type => motion, subtype => range, data => #{start => seconds(), duration => seconds()}}.

-record(dvr_info, {
  from = undefined :: seconds(),
  depth = undefined :: seconds(),
  ranges = undefined :: [dvr_range()],
  locks = undefined :: [dvr_range()],
  motion_log = undefined :: [dvr_motion_event()],
  brief_thumbnails = undefined :: [seconds()],
  bytes = undefined :: bytes(),
  disk_size = undefined :: bytes(),
  duration = undefined :: seconds()
}).




%  $$$$$$\              $$\     $$\       
% $$  __$$\             $$ |    $$ |      
% $$ /  $$ |$$\   $$\ $$$$$$\   $$$$$$$\  
% $$$$$$$$ |$$ |  $$ |\_$$  _|  $$  __$$\ 
% $$  __$$ |$$ |  $$ |  $$ |    $$ |  $$ |
% $$ |  $$ |$$ |  $$ |  $$ |$$\ $$ |  $$ |
% $$ |  $$ |\$$$$$$  |  \$$$$  |$$ |  $$ |
% \__|  \__| \______/    \____/ \__|  \__|










-record(auth_spec, {
  url = undefined :: auth_url(),
  domains = undefined :: [binary()],
  autogenerate_token = undefined :: boolean(),
  max_sessions = undefined :: non_neg_integer(),
  allowed_countries = undefined :: [iso3166()],
  disallowed_countries = undefined :: [iso3166()],
  soft_limitation = undefined :: boolean(),
  extra = undefined :: #{atom() => binary()}
}).







-type auth_backend_name() :: binary().

-record(auth_backend_url, {
  url = undefined :: binary(),
  extra = #{} :: #{atom() => binary()}
}).

-record(auth_backend_config, {
  name :: primary_key(auth_backend_name()),
  position = undefined :: sort_index(),
  allow_tokens = undefined :: [binary()],
  deny_tokens = undefined :: [binary()],
  allow_ips = undefined :: [binary()],
  deny_ips = undefined :: [binary()],
  allow_countries = undefined :: [binary()],
  deny_countries = undefined :: [binary()],
  allow_uas = undefined :: [binary()],
  deny_uas = undefined :: [binary()],
  allow_default = false :: boolean(),
  backends = undefined :: [#auth_backend_url{}]
}).



%  $$$$$$\                      $$\                 
% $$  __$$\                     $$ |                
% $$ /  \__| $$$$$$\   $$$$$$$\ $$$$$$$\   $$$$$$\  
% $$ |       \____$$\ $$  _____|$$  __$$\ $$  __$$\ 
% $$ |       $$$$$$$ |$$ /      $$ |  $$ |$$$$$$$$ |
% $$ |  $$\ $$  __$$ |$$ |      $$ |  $$ |$$   ____|
% \$$$$$$  |\$$$$$$$ |\$$$$$$$\ $$ |  $$ |\$$$$$$$\ 
%  \______/  \_______| \_______|\__|  \__| \_______|
                                                  
                                                  
                                                  


-record(cache_spec, {
  path = undefined :: disk_path(),
  reference = undefined :: cache_name(),
  misses = undefined :: non_neg_integer(),
  disk_limit = undefined :: non_neg_integer(),
  time_limit = undefined :: non_neg_integer()
}).




-record(segment_cache_spec, {
  path = undefined :: disk_path(),
  misses = undefined :: non_neg_integer(),
  disk_limit = undefined :: non_neg_integer(),
  time_limit = undefined :: seconds()
}).





-record(cache_config, {
  name :: primary_key(cache_name()),
  position = undefined :: sort_index(),
  path = undefined :: disk_path(),
  misses = undefined :: non_neg_integer(),
  disk_limit = undefined :: non_neg_integer(),
  time_limit = undefined :: non_neg_integer()
}).









%  $$$$$$\    $$\                $$\               
% $$  __$$\   $$ |               $$ |              
% $$ /  \__|$$$$$$\    $$$$$$\ $$$$$$\    $$$$$$$\ 
% \$$$$$$\  \_$$  _|   \____$$\\_$$  _|  $$  _____|
%  \____$$\   $$ |     $$$$$$$ | $$ |    \$$$$$$\  
% $$\   $$ |  $$ |$$\ $$  __$$ | $$ |$$\  \____$$\ 
% \$$$$$$  |  \$$$$  |\$$$$$$$ | \$$$$  |$$$$$$$  |
%  \______/    \____/  \_______|  \____/ \_______/ 


-record(memory_stats, {
  free = undefined :: bytes(),
  swap = undefined :: bytes(),
  swap_free = undefined :: bytes(),
  total = undefined :: bytes()
}).



-type pusher_status() :: session_status() | pending | retry | error.

-record(push_stats, {
  status = undefined :: pusher_status(),
  since = undefined :: seconds(),
  retries = undefined :: non_neg_integer(),
  last_error = undefined :: binary_value(),
  sent_bytes = undefined :: bytes()
}).


-record(stream_stats, {
  name = undefined :: stream_name(), % Это для префиксов
  alive = false :: boolean(),
  running = true :: boolean(),
  static = undefined :: boolean(),
  position = undefined :: sort_index(),
  remote = false :: boolean(),
  source_hostname = undefined :: server_name(),
  dvr_enabled = true :: boolean(),
  opened_at = undefined :: milliseconds(),
  lifetime = undefined :: milliseconds(),
  start_running_at = undefined :: milliseconds(),
  ts_delay = undefined :: milliseconds(),
  retry_count = undefined :: non_neg_integer(),
  client_count = undefined :: client_count(),
  last_dts = undefined :: ticks(),
  last_dts_at = undefined :: milliseconds(),
  last_access_at = undefined :: milliseconds(),
  input_error_rate = undefined :: non_neg_integer(),
  bytes_in = undefined :: bytes(),
  bytes_out = undefined :: bytes(),
  out_bandwidth = undefined :: speed(),
  bufferings = undefined :: non_neg_integer(),
  bitrate = undefined :: speed(),
  source_error = undefined :: binary(),
  url = undefined :: url(),
  id = undefined :: session_id(),
  source_id = undefined :: session_id(),
  current_agent_id = undefined :: agent_id(),
  agent_status = undefined :: connected | not_connected | rproxy_not_configured,
  server = undefined :: binary(),
  published_from = undefined :: binary(),
  published_via = undefined :: binary(),
  dvr_info = undefined :: #dvr_info{},
  dvr_only = false :: boolean(),
  media_info = undefined :: media_info(),
  input_media_info = undefined :: media_info(),
  push_stats = undefined :: #{url() => #push_stats{}},
  running_transcoder = false :: boolean(),

  dvr_replication = undefined :: percents(),
  dvr_replication_running = false :: boolean(),

  publish_enabled = false :: boolean(),
  transcoder_overloaded = false :: boolean()
}).


-type stream_stats() :: runtime(#stream_stats{}).


-record(vod_stats, {
  worker_count = undefined :: non_neg_integer(),
  client_count = undefined :: client_count(),
  prefix = undefined :: web_mount_point(),
  url = undefined :: disk_path(),
  bytes_in = undefined :: non_neg_integer(),
  bytes_out = undefined :: non_neg_integer(),
  name = undefined :: stream_name(),
  position = undefined :: sort_index()
}).

-type vod_stats() :: runtime(#vod_stats{}).



-record(transcoder_device_stats, {
  type = undefined :: transcoder_device(),
  id = undefined :: tc_deviceid(),
  name = undefined :: binary(),
  can_logo = undefined :: boolean(),
  can_interlace = undefined :: boolean(),
  can_internal = undefined :: boolean(),
  memTotal = undefined :: non_neg_integer(),
  memFree = undefined :: non_neg_integer(),
  memUsed = undefined :: non_neg_integer(),
  reconfig_support = undefined :: tc_reconfig_support(),
  gpu_temp = undefined :: non_neg_integer(),
  gpu_enc = undefined :: non_neg_integer(),
  gpu_dec = undefined :: non_neg_integer(),
  gpu_sm = undefined :: non_neg_integer()
}).

-type transcoder_device_stats() :: runtime(#transcoder_device_stats{}).


-record(stream_coder_stats, {
  bitstream_err_count = undefined :: non_neg_integer(),
  dec_err_count = undefined :: non_neg_integer(),
  dec_timing = undefined :: non_neg_integer(),
  enc_timing = undefined :: non_neg_integer(),
  encoders = undefined :: non_neg_integer(),
  fps_in = undefined :: non_neg_integer(),
  fps_out = undefined :: non_neg_integer(),
  mqlen = undefined :: non_neg_integer()
}).


-type stream_coder_stats() :: #stream_coder_stats{}.

-record(coder_status, {
  coder = undefined :: non_neg_integer(),
  boot_time = undefined :: non_neg_integer(),
  powered_at = undefined :: milliseconds(),
  dhcp_at = undefined :: milliseconds(),
  connected_at = undefined :: milliseconds(),
  updated_at = undefined :: seconds(),
  cpu_usage = undefined :: percents(),
  load = undefined :: percents(),
  mc_all = undefined :: non_neg_integer(),
  memory = undefined :: #memory_stats{},
  power = undefined :: non_neg_integer(),
  serial_number = undefined :: binary(),
  temperature = undefined :: celsius(),
  uptime = undefined :: milliseconds(),
  streams = undefined :: #{stream_name() => stream_coder_stats()},
  status = undefined :: unknown | rebooting | booting | working,
  reboot_allowed = true :: boolean()
}).
-type coder_status() :: runtime(#coder_status{}).


-type interface_name() :: binary().



% "runtime": {
%   "blinking": false,
%   "dns": [
%     "10.77.0.1"
%   ],
%   "gw": "10.77.0.1",
%   "ip": "10.77.1.150",
%   "mask": 23,
%   "mode": "dynamic",
%   "running": true,
%   "up": true
% },

-record(interface_stats, {
  blinking = false    :: boolean(),
  gw  = undefined     :: undefined | network_addr(),
  dns  = undefined    :: list(network_addr()),
  ip  = undefined     :: undefined | network_addr(),
  mask = undefined    :: undefined | network_mask(),
  mode = static       :: interface_mode(),
  running = false     :: boolean(),
  up = false          :: boolean()
}).

-type interface_stats() :: #interface_stats{} | undefined.

-record(interface_config, {
  gw  = undefined     :: network_addr(),
  dns  = undefined    :: list(network_addr()),
  ip  = undefined     :: network_addr(),
  mask = undefined    :: network_mask(),
  mode = static      :: interface_mode(),
  running = false     :: boolean()           
}).

-type interface_config() :: #interface_config{} | undefined.

-type network_interface() :: #{
  name => interface_name(),
  parameters => #{
    runtime => interface_stats(),
    settings => interface_config()
  }
}.

-record(chassis_dhcpd_status, {
  status = undefined :: failed|success,
  port = undefined :: network_port()
}).

-record(chassis_tftp_status, {
  port = undefined :: network_port(),
  root = undefined :: disk_path(),
  file_count = undefined :: non_neg_integer(),
  sent_bytes = undefined :: bytes()
}).

-record(chassis_ntpd_status, {
  port = undefined :: network_port()
}).


-record(chassis_status, {
  dhcp = undefined :: #chassis_dhcpd_status{},
  tftp = undefined :: #chassis_tftp_status{},
  ntpd = undefined :: #chassis_ntpd_status{},
  links = undefined :: #{interface_name() => network_interface()},
  coders = undefined :: [coder_status()],
  model = undefined :: binary(),
  version = undefined :: chassis_version(),
  serial_number = undefined :: binary(),
  hostname = undefined :: binary(),
  next_version = undefined :: chassis_version(),
  downloaded_versions = undefined :: [chassis_version()],
  hardware_id = undefined :: binary()
}).

-type chassis_status() :: #chassis_status{}.

-record(server_capabilities_stats, {
  url_publish = true :: boolean(),
  maintenance_page = false :: boolean(),
  server_error_page = false :: boolean(),
  need_to_init_credentials = false :: boolean()
}).


-type config_path_segment() :: atom() | binary().

-record(config_error_status, {
  error :: atom(),
  line = undefined :: non_neg_integer(),
  first_error_line = undefined :: non_neg_integer(),
  col = undefined :: non_neg_integer(),
  first_error_col = undefined :: non_neg_integer(),
  path = undefined :: [ config_path_segment() ],
  detail = undefined :: atom() | binary()
}).

%% SSL CERTIFICATES TYPES
-record(certificate_info, {
  domains     :: [ binary() ],
  not_before  = undefined :: non_neg_integer(),
  not_after   = undefined :: non_neg_integer(),
  issuer_name = undefined :: binary()
}).

-record(private_key_info, {
  match_certificate   :: boolean()  
}).

-record(ssl_certificate, {
  cert = undefined        :: #certificate_info{} | false,
  cacert = undefined      :: #certificate_info{} | false,
  key = undefined         :: #private_key_info{} 
}).


-record(server_stats, {
  version = undefined :: server_version(),
  build = undefined :: non_neg_integer(),
  streamer_status = undefined :: no_config | broken_config | license_failure | no_license | starting | running,
  http = undefined :: network_port(),
  https = undefined :: network_port(),
  admin_http = undefined :: network_port(),
  admin_https = undefined :: network_port(),
  rtsp = undefined :: network_port(),
  rtmp = undefined :: network_port(),
  turn = undefined :: network_port(),
  mysql = undefined :: network_port(),
  hostname = undefined :: binary(),
  dvr = undefined :: boolean(),
  license_txt = undefined :: binary(),
  license_type = undefined :: license_type(),
  id = undefined :: instance_id(),
  started_at = undefined :: seconds(),
  ad_injector = undefined :: boolean(),
  vsaas = undefined :: boolean(),
  vsaas_branding = undefined :: boolean(),
  vsaas_running = undefined :: boolean(),
  rproxy = undefined :: boolean(),
  rproxy_running = undefined :: boolean(),
  iptv = undefined :: boolean(),
  iptv_running = undefined :: boolean(),
  ffmpeg = undefined :: boolean(),
  transcoder = undefined :: boolean(),
  config2 = true :: boolean(),
  total_clients = undefined :: non_neg_integer(),
  total_streams = undefined :: non_neg_integer(),
  online_streams = undefined :: non_neg_integer(),
  opened_files = undefined :: non_neg_integer(),
  net = undefined :: [binary()],
  disks = undefined :: [binary()],
  bandwidth_usage = undefined :: percents(),
  total_bandwidth = undefined :: speed(),
  input_kbit = undefined :: speed(),
  output_kbit = undefined :: speed(),
  cpu_usage = undefined :: percents(),
  memory_usage = undefined :: percents(),
  scheduler_load = undefined :: percents(),
  uptime = undefined :: seconds(),
  partitions = undefined :: [ #{path => disk_path(), total_mb => megabytes(), usage => percents()}],
  auth_token = undefined :: auth_token(),
  meta = undefined :: server_meta(),
  capabilities = undefined :: #server_capabilities_stats{},
  transcoder_devices = undefined :: [transcoder_device_stats()],
  transcoder_defaults = undefined :: #{atom() => any()},
  next_version = undefined :: server_version(),
  status = undefined :: #{atom() => binary()},
  config_error = undefined :: #config_error_status{},
  chassis = undefined :: #{error => atom()} | chassis_status(),
  ssl_certificates = undefined :: #ssl_certificate{} | undefined
}).



% $$\       $$\                      
% $$ |      \__|                     
% $$ |      $$\ $$\    $$\  $$$$$$\  
% $$ |      $$ |\$$\  $$  |$$  __$$\ 
% $$ |      $$ | \$$\$$  / $$$$$$$$ |
% $$ |      $$ |  \$$$  /  $$   ____|
% $$$$$$$$\ $$ |   \$  /   \$$$$$$$\ 
% \________|\__|    \_/     \_______|


-define(WEBRTC_FIELDS, output_audio = undefined :: binary(),
  prefer_codec = undefined :: binary_value(),
  transport = undefined :: binary_value(),
  min_bitrate = undefined :: non_neg_integer(),
  webrtc_abr = undefined :: boolean(),
  abr_stepup = undefined :: non_neg_integer(),
  abr_correction = undefined :: non_neg_integer(),
  abr_loss_lower = undefined :: number(),
  abr_loss_upper = undefined :: number(),
  abr_stepdown = undefined :: number(),
  abr_mode = undefined :: non_neg_integer(),
  abr_debug = undefined :: non_neg_integer(),
  abr_cycles = undefined :: non_neg_integer()
).


-record(ttxt_descriptors, {
  page :: non_neg_integer(),
  lang :: binary_value(),
  type :: initial|subtitle|impaired
}).

-type stream_name() :: web_mount_point().
-type stream_source_id() :: binary().
-type group_name() :: binary().
-type live_prefix() :: web_mount_point().
-type dynamic_prefix() :: web_mount_point().

-type mpegts_lang_track() :: default | binary().


-record(stream_url, {
  url = undefined :: url(),
  source_timeout = undefined :: seconds() | false,
  priority = undefined :: non_neg_integer(),
  rtp = undefined :: udp,
  user_agent = undefined :: binary(),
  cluster_key = undefined :: binary(),
  buffer_length = undefined :: non_neg_integer(),
  timeout = undefined :: non_neg_integer(),
  frames_timeout = undefined :: non_neg_integer(),
  try_adts = undefined :: boolean(),
  connections = undefined :: non_neg_integer(),
  tracks = undefined :: binary_int(), %[non_neg_integer()],
  wait_rtcp = undefined :: boolean(),
  width = undefined :: non_neg_integer(),
  height = undefined :: non_neg_integer(),
  bitrate = undefined :: bitrate(),
  aac_ab = undefined :: binary_int(),
  via = undefined :: agent_url(),
  sar = undefined :: binary(),
  mbr = undefined :: binary(),
  max_bitrate = undefined :: bitrate(),

  % m4f(s)
  remote_dvr = undefined :: nochain | chain | none,


  % webrtc
  ?WEBRTC_FIELDS,

  % h323
  id = undefined :: binary(),
  video_bitrate = undefined :: speed(),
  audio_bitrate = undefined :: speed(),


  % hls
  buffer = undefined :: integer(),
  stall_check = undefined :: binary_boolean(),
  skip_stalled_check = undefined :: binary_boolean(),


  % mpegts
  programs = undefined :: [non_neg_integer()],
  pids = undefined :: [non_neg_integer()],
  allow_subtitles = undefined :: boolean(),
  no_fix_subs_dts = undefined :: boolean(),
  cc_check = undefined :: binary_value(), % #{content() => mpegts_cc_error_action()},
  subtitles = undefined :: binary_value(), % ocr_add | ocr_replace
  closed_captions = undefined :: #{binary() => binary()}, % #{<<"extract">> => <<"true">>, % TODO #14110: <<"608.1.lang">> => <<"en">>, <<"708.12.name">> => <<"Arabic">>, ...}
  dvbsubs_ocr = undefined :: binary_boolean(),
  scte35 = undefined :: boolean(),
  scte35_clock = undefined :: binary_int(),
  aac = undefined :: binary_boolean(),
  h264 = undefined :: binary_value(),
  headers = undefined :: #{atom() => binary()},
  languages = undefined :: #{mpegts_lang_track() => binary()},

  % mixer
  sync = undefined :: dts | realtime,
  audio_add = undefined :: integer(),

  % decklink
  mode = undefined :: binary_int(),
  ainput = undefined :: 1 | 2 | 3 | 4 | 5 | 6 | 7 | embedded | aes_ebu | analog | analog_xlr | analog_rca | microphone | headphones,
  vinput = undefined :: 1 | 2 | 3 | 4 | 5 | 6 | sdi | hdmi | optical_sdi | component | composite | s_video,
  vpts = undefined :: binary_value(), % audio | ref | video
  apts = undefined :: binary_value(), % audio | ref | video
  pixel = undefined :: binary_value(), % 8|10|rgb10|yuv10|rgb8|yuv8|rgb12,

  % aja
  channel = undefined :: non_neg_integer(),

  no_clients_reconnect_delay = undefined :: non_neg_integer(),

  % rtmp
  pageUrl = undefined :: url(),
  swfUrl = undefined :: url(),
  tcUrl = undefined :: url(),

  % v4l
  audio_device = undefined :: binary(),
  video_device = undefined :: binary(),
  vbi_device = undefined :: binary(),

  %teletext
  ttxt_descriptors = undefined :: [#ttxt_descriptors{}],

  %vbi
  vbi_threshold = undefined :: non_neg_integer(),
  vbi_debug = undefined :: boolean(),
  vbi_decoder = undefined :: binary_value()
}).


-record(cluster_ingest_config, {
  affinity = undefined :: binary(),
  capture_at = undefined :: binary()
}).

-record(backup_config, {
  file = undefined :: binary(),
  timeout = undefined :: non_neg_integer(),
  audio_timeout = undefined :: non_neg_integer(),
  video_timeout = undefined :: non_neg_integer(),
  transcode = undefined :: boolean(),
  dvr = undefined :: boolean()
}).


-record(stream_push_pid_config, {
  track = undefined :: pmt | non_neg_integer(),
  content = undefined :: video | audio | text |system
}).


-record(vbi_lines, {
  service = undefined :: vbi_service(),
  lines = undefined :: [vbi_line()]
}).


-record(stream_push_config, {
  url = undefined :: binary(),
  multicast_loop = undefined :: boolean(),
  vb = undefined :: bitrate(),
  bitrate = undefined :: bitrate(),
  pnr = undefined :: non_neg_integer(),
  pmt = undefined :: ts_pid(),
  pids = #{} :: #{ts_pid() => #stream_push_pid_config{}},
  dns = undefined :: binary(),
  retry_limit = undefined :: non_neg_integer(),
  retry_timeout = undefined :: non_neg_integer(),
  timeout = undefined :: non_neg_integer(),
  % decklink
  volume = undefined :: number(),
  deinterlace = undefined :: boolean(),
  fps = undefined :: binary_value(),
  size = undefined :: binary_value(),
  format = undefined :: binary_value(),
  vbi_lines = undefined :: [#vbi_lines{}],

  disabled = undefined :: boolean()
}).




-record(ad_inject_spec, {
  vast = undefined :: binary()
}).


-record(ad_schedule_spec, {
  alias = undefined :: binary(),
  schedule = undefined :: binary()
}).


-record(thumbnails_spec, {
  url = undefined :: binary(),
  enabled = undefined :: boolean()
}).


-record(play_protocols_spec, {
  whitelist = false :: boolean(),
  hls = undefined :: boolean(),
  dash = undefined :: boolean(),
  hds = undefined :: boolean(),
  rtmp = undefined :: boolean(),
  rtsp = undefined :: boolean(),
  m4s = undefined :: boolean(),
  mseld = undefined :: boolean(),
  mpegts = undefined :: boolean(),
  webrtc = undefined :: boolean(),
  mss = undefined :: boolean(),
  srt = undefined :: boolean(),
  shoutcast = undefined :: boolean(),
  m4f = undefined :: boolean(),
  mp4 = undefined :: boolean(),
  jpeg = undefined :: boolean()
}).


-record(silencedetect_spec, {
  duration = undefined :: non_neg_integer(),
  interval = undefined :: non_neg_integer(),
  noise :: decibels()
}).


-record(vision_spec, {
  gpu = undefined :: binary_int(),
  alg = undefined :: binary_value(),
  areas = undefined :: binary()
}).


-record(motion_detector_spec, {
  threshold = undefined :: binary_int(),
  tags = undefined :: [binary()],
  enabled = true :: boolean(),
  pull = undefined :: binary()
}).


-record(drm_spec, {
  vendor :: atom(), %aes128 | ezdrm | keyos | conax | pallycon | gsdrm ,
  keyserver = undefined :: binary(),
  user_keyserver = undefined :: binary(), % verimatrix
  resource_id = undefined :: drm_resource_id(),
  expires = undefined :: binary_int(),
  encryption = undefined :: binary(),
  aes_key = undefined :: binary(),
  content_id = undefined :: binary(),
  signer = undefined :: binary(),
  iv = undefined :: binary(),
  secret = undefined :: binary(),
  site = undefined :: binary(),
  fp = undefined :: binary(),
  account_id = undefined :: binary(),
  ic_host = undefined :: binary(),
  password = undefined :: binary(),
  user_name = undefined :: binary(),
  user = undefined :: binary(),
  keyseed = undefined :: binary(),
  key_id = undefined :: binary(),
  userkey = undefined :: binary(),
  enc_token = undefined :: binary(),
  user_path = undefined :: binary(),
  dump_url = undefined :: boolean(),
  save_template = undefined :: binary(),
  hls_ext_x_key_iv = undefined :: boolean(),
  systems = undefined :: [drm_system()]
}).



-record(subtitle_style, {
  align = undefined :: binary(),
  valign = undefined :: binary()
}).



-record(stream_mpegts_pids, {
  pmt = undefined :: non_neg_integer(),
  sdt = undefined :: non_neg_integer(),
  pcr = undefined :: non_neg_integer(),
  video = #{} :: #{non_neg_integer() => ts_pid()},
  audio = #{} :: #{non_neg_integer() => ts_pid()},
  text = #{} :: #{non_neg_integer() => ts_pid()}
}).


-record(web_logo_spec, {
  path = undefined :: logo_path(),
  height = undefined :: binary_int(),
  width = undefined :: binary_int(),
  left = undefined :: binary_int(),
  top = undefined :: binary_int(),
  bottom = undefined :: binary_int(),
  right = undefined :: binary_int()
}).


-define(COMMON_MEDIA_FIELDS, auth = undefined :: #auth_spec{},
  prepush = undefined :: boolean() | non_neg_integer(),
  cache = undefined :: #cache_spec{},
  protocols = undefined :: #play_protocols_spec{},
  epg_enabled = undefined :: boolean(),
  cmaf_enabled = undefined :: boolean(),
  transcoder = undefined :: #transcoder_opts{},
  backup = undefined :: #backup_config{},
  drm = undefined :: #drm_spec{},
  segment_count = undefined :: non_neg_integer(),
  segment_duration = undefined :: milliseconds(),
  provider = undefined :: binary(),
  record_input = undefined :: #record_input_spec{},
  dvr = undefined :: #stream_dvr_spec{},
  logo = undefined :: #web_logo_spec{},
  source_timeout = undefined :: seconds() | false,
  url_prefix = undefined :: url_prefix(),
  cluster_ingest = undefined :: #cluster_ingest_config{},
  cluster_key = undefined :: binary(),
  groups = undefined :: [group_name()],
  add_audio_only = undefined :: boolean(),
  pulse_off = undefined :: boolean(),
  max_bitrate = undefined :: bitrate(),
  thumbnails = undefined :: #thumbnails_spec{},
  frames_timeout = undefined :: non_neg_integer(),
  hls_scte35 = undefined :: false | aws | scte35,
  substyle = undefined :: #subtitle_style{},
  meta = #{} :: #{atom() => binary()}
).




-define(STREAM_FIELDS, title = undefined :: binary(),
  position = undefined :: sort_index(),
  ?COMMON_MEDIA_FIELDS,
  urls = undefined :: [#stream_url{}],
  disabled = undefined :: boolean(),
  pushes = #{} :: #{binary() => #stream_push_config{}},
  clients_timeout = undefined :: non_neg_integer() | boolean(),
  retry_limit = undefined :: non_neg_integer(),
  ad_inject = undefined :: #ad_inject_spec{},
  ad_schedule = undefined :: #ad_schedule_spec{},
  silencedetect = undefined :: #silencedetect_spec{},
  vision = undefined :: #vision_spec{},
  motion_detector = undefined :: #motion_detector_spec{},

  on_publish = undefined :: binary(),
  password = undefined :: binary(),
  rtp = undefined :: udp,
  publish_enabled = undefined :: boolean(),

  tracks = undefined :: [non_neg_integer()],
  mpegts_pids = undefined :: #stream_mpegts_pids{}
).





-record(stream_template, {
  name = undefined :: primary_key(stream_name()),
  ?STREAM_FIELDS
}).



-record(stream_config, {
  name = undefined :: primary_key(stream_name()),
  template = undefined :: stream_name(),
  stats = undefined :: stream_stats(),
  static = true :: boolean(),
  webrtc_abr = undefined :: boolean(),
  ?STREAM_FIELDS
}).


-record(live_config, {
  prefix = undefined :: primary_key(live_prefix()),
  streams = undefined :: #{stream_name() => stream_stats()},
  static = false :: boolean(),
  aac_ab = undefined :: binary_int(),
  ?STREAM_FIELDS,
  ?WEBRTC_FIELDS
}).


-type live() :: #live_config{}.
-type dynamic() :: #live_config{}.






% $$\    $$\                 $$\ 
% $$ |   $$ |                $$ |
% $$ |   $$ | $$$$$$\   $$$$$$$ |
% \$$\  $$  |$$  __$$\ $$  __$$ |
%  \$$\$$  / $$ /  $$ |$$ /  $$ |
%   \$$$  /  $$ |  $$ |$$ |  $$ |
%    \$  /   \$$$$$$  |\$$$$$$$ |
%     \_/     \______/  \_______|




-type vod_prefix() :: web_mount_point().




-record(vod_url, {
  url = undefined :: binary(),
  extra = #{} :: #{atom() => binary()}
}).



-record(vod_config, {
  prefix = undefined :: primary_key(web_mount_point()),
  position = undefined :: sort_index(),
  files = undefined :: #{binary() => vod_stats()},
  disabled = undefined :: boolean(),

  urls = [] :: [#vod_url{}],
  auth = undefined :: #auth_spec{},
  cache = undefined :: #cache_spec{},
  segment_cache = undefined :: #segment_cache_spec{},
  read_queue = undefined :: non_neg_integer(),
  max_readers = undefined :: non_neg_integer(),
  thumbnails = undefined :: #thumbnails_spec{},
  download = false :: boolean(),
  protocols = undefined :: #play_protocols_spec{},
  auto_mbr = false :: boolean(),
  url_prefix = undefined :: url_prefix(),
  segment_duration = undefined :: non_neg_integer(),
  logo = undefined :: #web_logo_spec{},
  timeout = undefined :: non_neg_integer(),
  drm = undefined :: #drm_spec{},
  max_sessions = undefined :: non_neg_integer(),
  pulse_off = false :: boolean(),
  add_audio_only = undefined :: boolean(),
  title = undefined :: binary(),
  provider = undefined :: binary(),
  meta = undefined :: #{atom() => binary()}
}).





%  $$$$$$\  $$\                       $$\                         
% $$  __$$\ $$ |                      $$ |                        
% $$ /  \__|$$ |$$\   $$\  $$$$$$$\ $$$$$$\    $$$$$$\   $$$$$$\  
% $$ |      $$ |$$ |  $$ |$$  _____|\_$$  _|  $$  __$$\ $$  __$$\ 
% $$ |      $$ |$$ |  $$ |\$$$$$$\    $$ |    $$$$$$$$ |$$ |  \__|
% $$ |  $$\ $$ |$$ |  $$ | \____$$\   $$ |$$\ $$   ____|$$ |      
% \$$$$$$  |$$ |\$$$$$$  |$$$$$$$  |  \$$$$  |\$$$$$$$\ $$ |      
%  \______/ \__| \______/ \_______/    \____/  \_______|\__|      


-type server_name() :: binary().


-record(remote_stats, {
  client_count = undefined :: client_count(),
  cpu_usage = undefined :: percents(),
  memory_usage = undefined :: percents(),
  output_bitrate = undefined :: speed(),
  stream_count = undefined :: integer(),
  bandwidth_usage = undefined :: percents()
}).


-record(peer_config, {
  name :: primary_key(server_name()),
  position = undefined :: sort_index(),
  stats = undefined :: #remote_stats{},
  public = undefined :: server_name(),
  fetch_timeout = undefined :: seconds(),
  stale_timeout = undefined :: seconds(),
  channel_limit = undefined :: non_neg_integer(),
  cpu_limit = undefined :: percents(),
  port = undefined :: network_port(),
  cluster_key = undefined :: binary(),
  max_bitrate = undefined :: speed()
}).




-type source_index() :: binary().

-record(source_group_config, {
  auth = undefined :: #auth_spec{},
  cache = undefined :: #cache_spec{},
  segment_duration = undefined :: milliseconds(),
  dvr = undefined :: #stream_dvr_spec{}
}).

-record(source_config, {
  prefix = undefined :: web_mount_point(),
  position = undefined :: sort_index(),
  stats = undefined :: #remote_stats{},
  streams = undefined :: #{stream_name() => stream_stats()},
  urls = [] :: [hostport()],
  only = undefined :: [stream_name()],
  except = undefined :: [stream_name()],
  ?COMMON_MEDIA_FIELDS,
  group_config = undefined :: #{group_name() => #source_group_config{}}
}).
-type source() :: #source_config{}.




%  $$$$$$\                       $$$$$$\  $$\           
% $$  __$$\                     $$  __$$\ \__|          
% $$ /  \__| $$$$$$\  $$$$$$$\  $$ /  \__|$$\  $$$$$$\  
% $$ |      $$  __$$\ $$  __$$\ $$$$\     $$ |$$  __$$\ 
% $$ |      $$ /  $$ |$$ |  $$ |$$  _|    $$ |$$ /  $$ |
% $$ |  $$\ $$ |  $$ |$$ |  $$ |$$ |      $$ |$$ |  $$ |
% \$$$$$$  |\$$$$$$  |$$ |  $$ |$$ |      $$ |\$$$$$$$ |
%  \______/  \______/ \__|  \__|\__|      \__| \____$$ |
%                                             $$\   $$ |
%                                             \$$$$$$  |
%                                              \______/ 



-type notify_name() :: binary().

-type notify_filter_value() :: [binary()].

-type notify_filter() :: #{
  atom() => notify_filter_value()
}.

-record(notify_config, {
  name :: primary_key(notify_name()),
  position = undefined :: sort_index(),
  sink = undefined :: binary(),
  args = #{} :: #{atom() => binary()},
  buffer = true :: boolean() | non_neg_integer(),
  only = undefined :: [notify_filter()],
  except = undefined :: [notify_filter()],
  owner = undefined :: binary(),
  throttle_delay = undefined :: seconds(),
  max_size = undefined :: bytes(),
  max_depth = undefined :: non_neg_integer(),
  resend_notifications_limit = undefined :: non_neg_integer(),
  resend_notifications_timeout = undefined :: seconds(),
  verbose = undefined :: loglevel(),
  extra = #{} :: #{atom() => binary()}
}).


-record(admin_credentials, {
  login = undefined :: binary(),
  password = undefined :: binary()
}).

-record(web_script_config, {
  prefix :: primary_key(web_mount_point()),
  path :: disk_path(),
  position = undefined :: sort_index(),
  extra = #{} :: #{atom() => binary()}
}).

-record(http_proxy_config, {
  prefix :: primary_key(web_mount_point()),
  url = undefined :: binary(),
  position = undefined :: sort_index()
}).



-type dvb_card_name() :: binary().

-type dvb_fec() :: '1_2' | '2_3' | '2_5' | '3_4' | '3_5' | '4_5' | '5_6' |
  '6_7' | '7_8' | '8_9' | '9_10'.
-type dvb_pilot() :: auto | on | off.

-record(dvb_card_stats, {
  adapter = undefined :: non_neg_integer(),  
  ber = undefined :: non_neg_integer(),
  strength_raw = undefined :: non_neg_integer(),
  strength = undefined :: non_neg_integer(),
  snr_raw = undefined :: non_neg_integer(),
  snr = undefined :: non_neg_integer(),
  has_signal = undefined :: boolean(), 
  has_carrier = undefined :: boolean(), 
  has_viterbi = undefined :: boolean(), 
  has_sync = undefined :: boolean(), 
  has_lock = undefined :: boolean(), 
  has_timedout = undefined :: boolean(), 
  has_reinit  = undefined :: boolean()
}).

-record(dvb_card_config, {
  name :: primary_key(dvb_card_name()),
  position = undefined :: sort_index(),
  adapter = undefined :: non_neg_integer(),
  frontend = undefined :: non_neg_integer(),
  frequency = undefined :: non_neg_integer(),
  symbol_rate = undefined :: non_neg_integer(),
  int_freq = undefined :: non_neg_integer(),
  high_band = undefined :: boolean(),
  polarization = undefined :: v|h|r|l,
  hierarchy = undefined :: '1'|'2'|'4'|none|auto,
  transmission_mode = undefined :: '1k' | '2k' | '8k' | '16k' | '32k' | 'c1' | 'c3780',
  guard_interval = undefined :: '1_128' | '1_32' | '1_16' |'1_8' | '1_4' | '19_128' | '19_256' | 'pn420' | 'pn595' | 'pn945',
  pilot = undefined :: dvb_pilot(),
  rolloff = undefined :: '35' | '25' | '20',
  bandwidth = undefined :: '8' | '7' | '6' | auto | '5' | '10' | '1_712',
  code_rate_lp = undefined :: dvb_fec(),
  code_rate_hp = undefined :: dvb_fec(),
  system = undefined :: dvbs | dvbs2 | dvbt | dvbt2 | dvbca | dvbcb | dvbcc | atsc | isdbt,
  modulation = undefined :: auto | qpsk | qam16 | qam32 | 
    qam64 | qam128 | qam256 | qam_auto | psk8 | apsk16 | apsk32 | vsb8 | vsb16 | dqpsk | qam4nr,
  disabled = undefined :: boolean(),
  comment = undefined :: binary(),
  stats = undefined :: #dvb_card_stats{}
}).



-type decklink_card_id() :: binary() | non_neg_integer() | test_fake. 

-record(decklink_config, {
  name :: primary_key(decklink_card_id()),
  position = undefined :: sort_index(),
  profile = undefined :: binary_value() % one_full | one_half | two_half | four_half
}).



-record(forward_ports_config, {
  handler = undefined :: atom(),
  options = undefined :: any()
}).

-type plugin_name() :: atom().
-record(plugin_config, {
  name :: primary_key(binary()),
  position = undefined :: sort_index(),


  % rproxy
  streampoint_key = undefined :: binary(),
  forward_ports = undefined :: #{network_port() => #forward_ports_config{}},
  endpoint_auth = undefined :: binary(),


  path = undefined :: #vod_url{},
  jpeg_vector_helper = undefined :: binary(),
  hw = undefined :: binary(),
  % backends = undefined :: [#{url => binary(), extra => #{atom() => binary()}}],


  loglevel = undefined :: debug | info | warning | error,
  port = undefined :: non_neg_integer(),
  database = undefined :: url(),
  mode = undefined :: binary_value(),
  workers = undefined :: non_neg_integer(),
  threads = undefined :: non_neg_integer(),
  endpoint = undefined :: enabled,
  base_url = undefined :: binary(),
  sentry_dsn = undefined :: url(),
  custom_auth = undefined :: url(),
  firebase_apikey = undefined :: binary(),
  firebase_endpoint = undefined :: binary(),
  firebase_project = undefined :: binary(),
  backup_path = undefined :: disk_path(),
  sql_log = undefined :: disk_path(),
  template_dir = undefined :: disk_path(),
  allow_multidomain = undefined :: boolean(),
  % deprecated
  file_sharing = undefined :: binary(),
  ext_event_handler = undefined :: binary(),
  backend_url = undefined :: url(),
  email_from = undefined :: [binary()],
  smtp_login = undefined :: binary(),
  smtp_opts = undefined :: binary(),
  smtp_server = undefined :: binary(),
  streamer = undefined :: binary(),
  workdir = undefined :: disk_path(),


  % dynafile
  cache = undefined :: #cache_spec{},
  transcoder = undefined :: #transcoder_opts{},

  % config_remote
  refresh_interval = undefined :: binary_int(), 
  valid_time = undefined :: binary_int(),
  keep_time = undefined :: binary_int(),
  backends = undefined :: [#vod_url{}],
  default = undefined :: binary(),

  % chassis:
  api_port = undefined :: binary(),
  auto_reboot = undefined :: boolean(),
  dhcpd_iface = undefined :: binary(),
  dhcpd_port = undefined :: binary(),
  dhcpd_port_dest = undefined :: binary(),
  lcd = undefined :: boolean(),
  ntpd_iface = undefined :: binary(),
  tftp_port = undefined :: binary(),
  tftp_root = undefined :: disk_path(),
  update_channel = undefined :: binary(),
  hal = undefined :: atom(),
  ntpd_port = undefined :: binary(),
  firmware_host = undefined :: url(),
  firmware_version = undefined :: binary(),
  firmware_boot_dir = undefined :: disk_path(),

  % camera alarm
  listen = undefined :: binary(),

  extra = undefined :: #{atom() => binary()}
}).

-type cidr() :: binary().


-type balancer_name() ::binary().
-record(balancer_config, {
  peers = #{} :: #{server_name() => #peer_config{}}
}).



-record(server_config, {
  version = undefined :: config_version(),
  backup_on_save = undefined :: atom(),
  http = [] :: [listen_spec()],
  https = [] :: [listen_spec()],
  admin_http = [] :: [listen_spec()],
  admin_https = [] :: [listen_spec()],

  geoip = undefined :: disk_path(),

  rtsp = undefined :: listen_spec(),
  rtsps = undefined :: listen_spec(),
  turn = undefined :: listen_spec(),
  rtmp = undefined :: listen_spec(),
  rtmps = undefined :: listen_spec(),
  mysql = undefined :: listen_spec(),
  snmp = undefined :: listen_spec(),
  sip = undefined :: listen_spec(),
  srt = undefined :: listen_spec(),
  meta = undefined :: server_meta(),
  loglevel = undefined :: loglevel(),
  nvidia_monitor = false :: boolean(),
  total_bandwidth = undefined :: speed(),
  wwwroot = undefined :: disk_path(),
  log_requests = false :: boolean(),
  auth_token = undefined :: auth_token(),
  max_sessions = undefined :: deprecated(non_neg_integer()),
  auth = undefined :: #auth_spec{},
  %autogenerate_token = true :: boolean(), % Это пойдет в auth
  notifies = #{} :: #{notify_name() => #notify_config{}},
  aliaser = undefined :: aliaser_url(),
  pulsedb = undefined :: disk_path(),
  session_log = undefined :: disk_path(),
  cluster_key = undefined :: binary(),
  url_prefix = undefined :: url_prefix(),
  view_auth = undefined :: #admin_credentials{},
  edit_auth = undefined :: #admin_credentials{},
  init_script = undefined :: disk_path(),
  web_scripts = #{} :: #{web_mount_point() => #web_script_config{}},
  http_proxies = #{} :: #{web_mount_point() => #http_proxy_config{}},
  api_allowed_from = undefined :: [cidr()],
  cdnproxy = false :: boolean(),
  license_server = undefined :: [binary()],
  auth_backends = #{} :: #{auth_backend_name() => #auth_backend_config{}},

  dvrs = #{} :: #{dvr_name() => #dvr_config{}},
  peers = #{} :: #{server_name() => #peer_config{}},
  sources = #{} :: #{source_index() => #source_config{}},
  transponders = #{} :: #{transponder_name() => #transponder_config{}},
  templates = #{} :: #{stream_name() => #stream_template{}},
  streams = #{} :: #{stream_name() => #stream_config{}},
  ephemerals = #{} :: #{stream_name() => #stream_config{}},
  dynamics = #{} :: #{dynamic_prefix() => dynamic()},
  lives = #{} :: #{live_prefix() => live()},
  caches = #{} :: #{cache_name() => #cache_config{}},
  vods = #{} :: #{vod_prefix() => #vod_config{}},
  dvb_cards = #{} :: #{dvb_card_name() => #dvb_card_config{}},
  decklinks = #{} :: #{decklink_card_id() => #decklink_config{}},
  plugins = #{} :: #{plugin_name() => #plugin_config{}},
  balancers = #{} :: #{balancer_name() => #balancer_config{}},
  includes = [] :: [disk_path()],

  stats = undefined :: #server_stats{},
  chassis_interface = undefined :: #{interface_name() => interface_config()}
}).

-type server_config_root() :: #server_config{}.

-record(config_validation_response, {
  config_error          :: atom(),
  detail = undefined    :: any(),
  reason = undefined    :: atom(),
  detail2 = undefined   :: atom(),
  path = undefined      :: any(),
  unparsed = undefined  :: any(),
  rest = undefined      :: any(),
  col = undefined       :: any(),
  line = undefined      :: any(),
  what = undefined      :: any()
}).

-type config_validation_response() :: #config_validation_response{}.

