

-type event_loglevel() :: loglevel().
-type event_logformat() :: [ any() ].

-define(DEFAULT_FIELDS,
  event = undefined :: atom(),
  utc_ms = undefined :: milliseconds(),
  event_id = undefined :: non_neg_integer(),
  application = undefined :: atom(),
  module = undefined :: atom(),
  line = undefined :: non_neg_integer(),
  date = undefined :: binary(),
  time = undefined :: binary(),
  message = undefined :: binary(),
  pid = undefined :: pid()
).

-define(DEFAULT_MEDIA_FIELDS,
  ?DEFAULT_FIELDS,
  media = undefined :: stream_name(),
  submedia = undefined :: non_neg_integer() | stream_name()
).

-define(DEFAULT_AGENT_FIELDS, 
  ?DEFAULT_MEDIA_FIELDS,
  agent_id = undefined :: agent_id(),
  ip = undefined :: network_addr(),
  local_ip = undefined :: network_addr()
).

-record(text,{
  ?DEFAULT_FIELDS,
  loglevel = none :: event_loglevel(),
  format = undefined :: event_logformat()
}). % Fake event for plain text routing

-record(crashed,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["server ",server_state, " crashed on message ", last_msg, " with reason ",reason] :: event_logformat(),
  server_state = undefined :: any(),
  last_msg = undefined :: any(),
  reason = undefined :: any()
}).



-record(server_starting,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(), 
  format = ["server ",full_version," is starting"] :: event_logformat(),
  version = undefined :: server_version(),
  full_version = undefined :: server_version()
}).


-record(change_server_id, {
  ?DEFAULT_FIELDS,
  loglevel = alert :: event_loglevel(), 
  old :: server_id(),
  format = ["change server_id requested from ",old] :: event_logformat()  
}).

-record(server_started,{
  ?DEFAULT_FIELDS,
  loglevel = alert :: event_loglevel(),
  format = ["server ",version," started. previous state ", previous_state] :: event_logformat(),
  previous_state = undefined :: previous_server_state(),
  instance_id :: instance_id(),
  version :: server_version()
}).



-record(presence_connected,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["presence server connected with ",flush," flushed"] :: event_logformat(),
  flush = undefined :: non_neg_integer(),
  ip = undefined :: network_addr()
}).

-record(presence_disconnected,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  reason = undefined :: atom(),
  format = ["presence server disconnected: ",reason] :: event_logformat(),
  ip = undefined :: network_addr()
}).


-record(server_stopping,{
  ?DEFAULT_FIELDS,
  loglevel = alert :: event_loglevel(),
  format = ["server ",full_version, " is shutting down after ", uptime, " seconds of work"] :: event_logformat(),
  version :: server_version(),
  full_version :: server_version(),
  uptime :: seconds()
}).

-record(too_many_streams,{
  ?DEFAULT_FIELDS,
  loglevel = alert :: event_loglevel(),
  format = ["Number of streams is bigger than license limit: ",limit] :: event_logformat(),
  limit = undefined :: non_neg_integer()
}).

-record(listener_start,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "start ",
  {proto, [proto, " "], ""},
  "listener on ",
  {ip, [ip,":"], ""},
  port, " ", {inet, [inet], ["ipv4"]}
  ] :: event_logformat(),
  proto :: listen_protocol(),
  port :: listen_spec(),
  ip = undefined :: network_addr(),
  inet = ipv4 :: network_version()
}).

-record(listener_stop,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "stop ",
  {proto, [proto, " "], ""},
  "listener on ",
  {ip, [ip,":"], ""},
  port, " ", {inet, [inet], ["ipv4"]}
  ] :: event_logformat(),
  proto :: listen_protocol(),
  port :: network_port(),
  ip = undefined :: network_addr(),
  inet = ipv4 :: network_version()
}).

-record(listener_failure,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "failure starting ",
  {proto, [proto, " "], ""},
  "listener on ",
  {ip, [ip,":"], ""},
  port, ": ",
  {error, [error, " "], ""},
  "(", error_code, ")"
  ] :: event_logformat(),
  proto :: listen_protocol(),
  port :: network_port(),
  ip = undefined :: network_addr(),
  inet = ipv4 :: network_version(),
  error_code = undefined :: binary_value(),
  error = undefined :: any()
}).

-record(resource_overloaded,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["resource ",resource," overloaded, threshold: ", threshold] :: event_logformat(),
  resource :: binary(),
  threshold :: number()
}).

-record(resource_not_overloaded,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["resource ",resource," consumption normalized, threshold: ", threshold] :: event_logformat(),
  resource :: binary(),
  threshold :: number()
}).


-record(config_reloaded,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["config reloaded in ", duration, "ms"] :: event_logformat(),
  version :: config_version(),
  duration = undefined :: milliseconds()
}).





-record(web_request,{
  ?DEFAULT_FIELDS,
  loglevel = none :: event_loglevel(),
  format = [date," ",time," ",code," ",runtime," ",size," ",path," ",session_id," ",ip,
    {disk_time, [" disk=",disk_time], ""}] :: event_logformat(),
  runtime = undefined :: milliseconds(),
  path = undefined :: binary(),
  session_id = undefined :: binary(),
  code = undefined :: non_neg_integer(),
  size = undefined :: non_neg_integer(),
  ip = undefined :: network_addr(),
  media = undefined :: stream_name(),
  open_time = undefined :: microseconds(),
  disk_time = undefined :: microseconds(),
  prepare_time = undefined :: microseconds(),
  fetch_time = undefined :: microseconds(),
  remote_time = undefined :: microseconds()
}).

-record(file_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [access,":open(",url,")"] :: event_logformat(),
  access = file :: file_access_method(),
  url = undefined :: url(),
  full_url = undefined :: url(),
  client_count = undefined :: integer()
}).

-record(file_closed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["file closed"] :: event_logformat()
}).


-record(stream_stop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = undefined :: event_logformat()
}).

-record(stream_reconnect_source_limit,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["exit due to retry limit ",count,"/", limit] :: event_logformat(),
  count = undefined :: non_neg_integer(),
  limit = undefined :: non_neg_integer()
}).


% deprecated
-record(stream_stopped,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stream stopped"] :: event_logformat(),
  client_count = undefined :: integer(),
  bytes_in = undefined :: bytes(),
  bytes_out = undefined :: bytes(),
  last_dts = undefined :: ticks(),
  lifetime = undefined :: milliseconds()
}).

-record(stream_ondemand_stopping,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["ondemand stream is stopping: no source"] :: event_logformat(),
  ts_delay = undefined :: milliseconds(),
  source_timeout = undefined :: seconds()
}).

-record(stream_reconfigure,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["update options from\n",old_options,"\nto\n",new_options] :: event_logformat(),
  streams = [] :: [stream_name()],
  old_options = undefined :: any(),
  new_options = undefined :: any()
}).

-record(stream_reconfigured,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = undefined :: event_logformat()
}).

-record(stream_motion_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Motion start: ", motion_id, " at: ", utc] :: event_logformat(),
  ip = undefined :: network_addr(),
  motion_id = undefined :: motion_id(),
  utc = undefined :: seconds(),
  motion_data = undefined :: any(),
  img = undefined :: jpeg_base64()
}).
-record(stream_motion_stopped,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Motion stop: ", motion_id, " duration: ", duration] :: event_logformat(),
  ip = undefined :: network_addr(),
  motion_id = undefined :: motion_id(),
  utc = undefined :: seconds(),
  motion_data = undefined :: any(),
  duration = undefined :: seconds()  
}).

-record(video_activity,{
  ?DEFAULT_MEDIA_FIELDS,
  node = undefined :: binary(),
  loglevel = debug :: event_loglevel(),
  format = ["video_activity: ", algorithm] :: event_logformat(),
  id = undefined :: uuid(),
  utc = undefined :: seconds(),
  algorithm = undefined :: binary_value(), % plate_detector | face_detector
  payload = undefined :: #{binary_value() => any()},
  img = undefined :: jpeg_base64(),
  vector = undefined :: binary()
}).

-record(video_activity_stats,{
  ?DEFAULT_FIELDS,
  node = undefined :: binary(),
  loglevel = debug :: event_loglevel(),
  format = ["video_activity_stats ", node, ": ", payload] :: event_logformat(),
  payload = undefined :: #{stream_name() => any()}
}).

-record(stream_helper_died,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["died ", mode, " helper ", id] :: event_logformat(),
  mode = undefined :: temporary | permanent,
  id = undefined :: any(),
  mfa = undefined :: any(),
  reason = undefined :: any()
}).



-record(cluster_ingest_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Cluster ingest starts stream"] :: event_logformat()
}).
-record(cluster_ingest_stop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Cluster ingest stop stream, master is on ", host] :: event_logformat(),
  host = undefined :: server_name()
}).
-record(cluster_server_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Failed to connect to ",server_name, ": ", error] :: event_logformat(),
  server_name = undefined :: server_name(),
  error = undefined :: any()
}).
-record(cluster_server_offline,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Server offline ",server_name] :: event_logformat(),
  server_name = undefined :: server_name()
}).
-record(cluster_server_online,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Server online ",server_name] :: event_logformat(),
  server_name = undefined :: server_name()
}).


-record(transcoder_overloaded,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Transcoder overloaded. Load: ", load] :: event_logformat(),
  load = undefined :: non_neg_integer()
}).
-record(transcoder_not_overloaded,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Transcoder returned to normal mode. Load: ", load] :: event_logformat(),
  load = undefined :: non_neg_integer()
}).
-record(transcoder_fin_overloaded,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Transcoder completely overloaded. Stop."] :: event_logformat()
}).


-record(transcoder_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["transcoder opened"] :: event_logformat(),
  hw = undefined :: transcoder_device()
}).
-record(transcoder_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["transcoding started"] :: event_logformat(),
  hw = undefined :: transcoder_device()
}).
-record(transcoder_start_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["transcoded failed to start: ", error] :: event_logformat(),
  error = undefined :: any()
}).
-record(transcoder_stopped,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["transcoding stopped",{reason, [": ",reason], ""}] :: event_logformat(),
  reason = undefined :: reconfiguration | binary()
}).
-record(transcoder_timeout,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["transcoder timeout"] :: event_logformat()
}).

-record(transcoder_media_info,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = [method, " due to media info change ", old_media_info, " ", new_media_info] :: event_logformat(),
  method = undefined :: reconfigure | restart,
  old_media_info = undefined :: binary(),
  new_media_info = undefined :: binary()
}).


-record(source_switch,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["switch source to ", url] :: event_logformat(),
  url = undefined :: url()
}).

-record(stream_backup,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stream backup playing ", file, " due to ", reason] :: event_logformat(),
  file = undefined :: url(),
  reason = undefined :: timeout | video_timeout | audio_timeout | no_sources
}).

-record(stream_backup_stop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stream backup stop playing ", file] :: event_logformat(),
  file = undefined :: url()
}).




-record(stream_kill_old_source,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "kill source",
  {oldproto, [" ", oldproto], ""},
  {oldip, [" ", oldip], ""},
  {oldpid, [" ", oldpid], ""},
  {reason, [": ", reason], ""}
  ] :: event_logformat(),
  oldpid = undefined :: pid(),
  oldname = undefined :: any(),
  oldstarted_at = undefined :: seconds(),
  oldip = undefined :: network_addr(),
  reason = undefined :: any()
}).


-record(stream_media_info,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  {old_media_info, ["was ", old_media_info, " now "], ""},
  "media_info ", media_info
  ] :: event_logformat(),
  media_info = undefined :: binary(),
  old_media_info = undefined :: binary()
}).




-record(transponder_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["starting transponder"] :: event_logformat()
}).
-record(transponder_source_added,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["adding ",stream," to transponder"] :: event_logformat(),
  stream = undefined :: stream_name()
}).
-record(transponder_source_removed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["remove ",stream," from transponder"] :: event_logformat(),
  stream = undefined :: stream_name()
}).
-record(transponder_got_epg,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["transponder got new epg"] :: event_logformat()
}).
-record(transponder_uploaded_xmltv,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  filename :: binary(),
  disk_path :: binary(),
  format = ["uploaded xmltv ", filename, ", saved to ", disk_path] :: event_logformat()
}).
-record(transponder_deleted_xmltv,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  disk_path :: binary(),
  format = ["deleted xmltv file ", disk_path] :: event_logformat()
}).



-record(adinjected,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "showed ", advert_type, " ", advert
  ] :: event_logformat(),
  advert = undefined :: binary(),
  advert_type = undefined :: preroll | midroll,
  interval = undefined :: ticks(),
  last_inject_at = undefined :: ticks(),
  proto = undefined :: hls | rtmp | mpegts
}).


-record(stream_jpeg,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = none :: event_loglevel(),
  format = ["jpeg ", utc] :: event_logformat(),
  utc = undefined :: seconds()
}).
-record(stream_force_close_gop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Forced close of too big gop: gap between frames ",gap," ms"] :: event_logformat(),
  gap = undefined :: milliseconds()
}).
-record(stream_force_open_gop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Had to start gop from ",flavor,"(",codec,
  ") because duration is ",duration," ms, next_id=",next_id,", goplen=",gop_length,", gop_age=",gop_age] :: event_logformat(),
  flavor = undefined :: frame_flavor(),
  codec = undefined :: frame_codec(),
  duration = undefined :: milliseconds(),
  next_id = undefined :: any(),
  gop_length = undefined :: non_neg_integer(),
  gop_age = undefined :: milliseconds()
}).
-record(stream_rt_sync,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["stream_rt_resync dts:",dts," start_dts:", start_dts,
  " stream_delta:",stream_delta," rt_delta:",rt_delta] :: event_logformat(),
  dts = undefined :: milliseconds(),
  start_dts = undefined :: milliseconds(),
  stream_delta = undefined :: milliseconds(),
  rt_delta = undefined :: milliseconds()
}).

-record(stream_broken_source,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["consider broken source with gop length ",length, " duration ",duration] :: event_logformat(),
  length = undefined :: non_neg_integer(),
  duration = undefined :: milliseconds()
}).

-record(stream_consumer_overloaded,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
    consumer_name," ",consumer_pid," overloaded with ", message_count," and status ", consumer_status, " messages:\n", stacktrace
  ] :: event_logformat(),
  message_count = undefined :: non_neg_integer(),
  stacktrace = undefined :: any(),
  consumer_pid = undefined :: pid(),
  consumer_name = undefined :: any(),
  consumer_status = undefined :: any()
}).



-record(dvr_storage_started,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["dvr storage ", name, " started", {no_index, " with no_index", ""} ] :: event_logformat(),
  name = undefined :: dvr_name(),
  root = undefined :: disk_path(),
  no_index = undefined :: boolean()
}).
-record(dvr_raid_db_loaded,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["dvr storage ", name, " raid DB loaded in ", load_time, " seconds"] :: event_logformat(),
  name = undefined :: dvr_name(),
  load_time = undefined :: seconds()
}).
-record(dvr_storage_error,{
  ?DEFAULT_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["dvr storage ", root, " error: ", error] :: event_logformat(),
  root = undefined :: disk_path(),
  error = undefined :: any()
}).


-record(dvr_info_updated,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = none :: event_loglevel(),
  format = ["DVR info updated, bytes=",disk_size] :: event_logformat(),
  disk_size = undefined :: bytes()
}).

-record(dvr_open_session,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR session open type=", access, " from=", from, " to=", to] :: event_logformat(),
  ip = undefined :: network_addr(),
  access = undefined :: dvr_live | dvr_file,
  from = undefined :: seconds(),
  to = undefined :: seconds() | now
}).
-record(dvr_new_fragment,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = none :: event_loglevel(),
  format = ["segment ", utc] :: event_logformat(),
  utc = undefined :: seconds(),
  duration = undefined :: milliseconds()
}).
-record(dvr_deleted_fragments,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = none :: event_loglevel(),
  format = ["deleted segments "] :: event_logformat(),
  path = undefined :: disk_path(),
  utcs = undefined :: [non_neg_integer()]
}).
-record(dvr_new_blob,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["hour blob ", path, {disk_size, [", bytes=", disk_size], ""}] :: event_logformat(),
  disk_size = undefined :: bytes(),
  path = undefined :: disk_path()
}).
-record(dvr_collapsed_write,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["collapsed write of ",count," segments"] :: event_logformat(),
  count = undefined :: non_neg_integer()
}).
-record(dvr_drop_write,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["drop write of ",path] :: event_logformat(),
  path = undefined :: binary()
}).

-record(dvr_locked,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR locked ",from, "-",duration] :: event_logformat(),
  from = undefined :: seconds(),
  duration = undefined :: milliseconds()
}).
-record(dvr_unlocked,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR unlocked ",from, "-",duration] :: event_logformat(),
  from = undefined :: seconds(),
  duration = undefined :: milliseconds()
}).

-record(dvr_copy_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR upload started ", local, " to ", clear_remote] :: event_logformat(),
  local = undefined :: disk_path(),
  remote = undefined :: url(),
  clear_remote = undefined :: url()
}).
-record(dvr_copy_ready,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR upload ",bytes," bytes ready from ", local, " during ",duration," seconds"] :: event_logformat(),
  local = undefined :: disk_path(),
  remote = undefined :: url(),
  duration = undefined :: seconds(),
  bytes = undefined :: bytes()
}).
-record(dvr_copy_already,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR already uploaded ",local, " with ",bytes," bytes"] :: event_logformat(),
  local = undefined :: disk_path(),
  remote = undefined :: url(),
  bytes = undefined :: bytes()
}).
-record(dvr_copy_failed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR upload fails: ", local, " ",reason ] :: event_logformat(),
  local = undefined :: disk_path(),
  remote = undefined :: url(),
  reason = undefined :: any(),
  local_size = undefined :: bytes(),
  remote_size = undefined :: bytes(),
  duration = undefined :: seconds()
}).

-record(dvr_blob_deleted,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR delete ",path," reason: ", reason_text, {disk_size, [", bytes=", disk_size], ""}] :: event_logformat(),
  disk_size = undefined :: bytes(),
  reason = undefined :: any(),
  reason_text = undefined :: binary(),
  path = undefined :: disk_path(),
  blob = undefined :: dvr_blob_path()
}).
-record(dvr_blob_clean_locked,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR cleans locked ",path," reason: ", reason_text] :: event_logformat(),
  segments_keep = undefined :: non_neg_integer(),
  segments_drop = undefined :: non_neg_integer(),
  reason = undefined :: any(),
  reason_text = undefined :: binary(),
  path = undefined :: disk_path(),
  blob = undefined :: dvr_blob_path()
}).
-record(dvr_blob_cleaner_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["DVR cannot clean: ", error] :: event_logformat(),
  error = undefined :: binary()
}).
-record(dvr_blob_skip_clean_lock,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["DVR locked blob clean skipped ", path] :: event_logformat(),
  reason = undefined :: any(),
  reason_text = undefined :: binary(),
  path = undefined :: disk_path(),
  blob = undefined :: dvr_blob_path()
}).

-record(dvr_segment_already_written,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR segment ",path," already written with ",bytes," bytes"] :: event_logformat(),
  path = undefined :: segment_path(),
  bytes = undefined :: non_neg_integer(),
  skip = undefined :: atom()
}).


-record(dvr_replication_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR replication started. Going to replicate ", blob_count, " blobs"] :: event_logformat(),
  blob_count = undefined :: non_neg_integer()
}).
-record(dvr_hour_replication_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["DVR replicating hour ", hour, ". Need to download ", segment_count, " of ", total_count, " segments"] :: event_logformat(),
  hour = undefined :: dvr_blob_path(),
  url = undefined :: url(),
  segment_count = undefined :: non_neg_integer(),
  total_count = undefined :: non_neg_integer(),
  local_count = undefined :: non_neg_integer()
}).
-record(dvr_hour_replication_done,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["DVR hour ", hour, " replication result: ", segments_ok, " ok, ", segments_failed, " failed"] :: event_logformat(),
  hour = undefined :: dvr_blob_path(),
  segments_ok = undefined :: non_neg_integer(),
  segments_failed = undefined :: non_neg_integer()
}).
-record(dvr_replication_progress,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["DVR replication in progress: ", progress, "%"] :: event_logformat(),
  progress = undefined :: percents()
}).
-record(dvr_replication_done,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DVR replication done with ", total_count, " segments in ", hour_count, " hours"] :: event_logformat(),
  total_count = undefined :: non_neg_integer(),
  hour_count = undefined :: non_neg_integer()
}).

-record(scte35,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["pnr: ",program," scte35: ", info] :: event_logformat()
}).




-record(agent_connected,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["connected from ip ",ip," with local ip ",local_ip] :: event_logformat()
}).
-record(agent_disconnected,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["disconnected", {error, [" with error: ",error],""}] :: event_logformat(),
  error = undefined :: any()
}).
-record(agent_too_many_endpoint_reconnects,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Too many reconnects to streampoint"] :: event_logformat()
}).
-record(agent_endpoint_rejected,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["endpoint auth rejected ",version," with code ",reject_code] :: event_logformat(),
  version = undefined :: agent_version(),
  auth_hash = undefined :: binary(),
  reject_code = undefined :: non_neg_integer()
}).
-record(agent_sent_to_streampoint,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["agent is sent to ",host," after ",reconnects," reconnects"] :: event_logformat(),
  host = undefined :: url(),
  reconnects = undefined :: non_neg_integer()
}).
-record(agent_streampoint_old_auth,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["agent ",agent_id," with old streampoint auth"] :: event_logformat()
}).
-record(agent_streampoint_connected,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["agent streampoint connected version ", version, " with caps ", caps] :: event_logformat(),
  version = undefined :: agent_version(),
  capabilities = undefined :: [agent_capability()]
}).
-record(streampoint_start_forward_proxy,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["streampoint start forward proxy on ",port ,", reqid ", reqid] :: event_logformat(),
  port = undefined :: network_port(),
  reqid = undefined :: agent_reqid()
}).
-record(streampoint_forward_connect,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["streampoint forward proxy port ", port, " connected from ", peer_address] :: event_logformat(),
  port = undefined :: network_port(),
  peer_address = undefined :: network_addr()
}).
-record(streampoint_connect_closed_efbig,{
  ?DEFAULT_AGENT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["agent dropped data connection due to buffer overrun, reqid ", reqid] :: event_logformat(),
  reqid = undefined :: agent_reqid(),
  code = undefined :: non_neg_integer()
}).

-record(pulsedb_start_writing,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["start writing pulse to ", path] :: event_logformat(),
  path = undefined :: disk_path()
}).
-record(pulsedb_wal_write_delay,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["delay in writing", {metric, [" ",metric], ""}, " to wal: ",stack] :: event_logformat(),
  stack = undefined :: any(),
  metric = undefined :: binary()
}).
-record(pulsedb_failure_opening_config,{
  ?DEFAULT_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["failed to open config ",path,": ",error] :: event_logformat(),
  path = undefined :: disk_path(),
  error = undefined :: any()
}).
-record(pulsedb_open_wal,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["open WAL ",path] :: event_logformat(),
  path = undefined :: disk_path()
}).
-record(pulsedb_error_opening_wal,{
  ?DEFAULT_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["failed to open WAL ",path,": ",error] :: event_logformat(),
  path = undefined :: disk_path(),
  error = undefined :: any()
}).
-record(pulsedb_error_collecting,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Error collecting ",name,": ",negative] :: event_logformat(),
  name = undefined :: binary(),
  negative = undefined :: any()
}).
-record(pulsedb_capture_drop,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["capture_drop ",name,": ",prev_utc," vs ",next_utc] :: event_logformat(),
  name = undefined :: binary(),
  prev_utc = undefined :: seconds(),
  next_utc = undefined :: seconds()
}).
-record(pulsedb_collect_failure,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Failed to collect pulse ",name,": ",error,"\nremote stacktrace:\n",remote_error,"\n"] :: event_logformat(),
  name = undefined :: binary(),
  error = undefined :: any(),
  remote = undefined :: any()
}).
-record(pulsedb_tick_too_long,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["tick took ",duration," us"] :: event_logformat(),
  duration = undefined :: milliseconds()
}).
-record(pulsedb_wal_restore_failure,{
  ?DEFAULT_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["failed to restor from WAL ",path,": ",error] :: event_logformat(),
  path = undefined :: disk_path(),
  error = undefined :: any()
}).
-record(pulsedb_wal_restore_ready,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["restored from WAL ",path," with ",metrics," metrics and ",entries," entries during ",duration," ms"] :: event_logformat(),
  path = undefined :: disk_path(),
  metrics = undefined :: non_neg_integer(),
  entries = undefined :: non_neg_integer(),
  duration = undefined :: milliseconds()
}).





-record(disk_cache_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Start disk cache"] :: event_logformat()
}).
-record(disk_cache_update,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Update options for disk cache"] :: event_logformat()
}).
-record(disk_cache_stop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Stop disk cache"] :: event_logformat()
}).
-record(disk_cache_loader_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Start disk cache loader"] :: event_logformat()
}).
-record(disk_cache_loader_progress,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Loader still running: processed ",dircount," dirs, ",filecount," files, current path ",current_dir] :: event_logformat(),
  dircount = undefined :: non_neg_integer(),
  filecount = undefined :: non_neg_integer(),
  duration = undefined :: milliseconds(),
  current_dir = undefined :: disk_path()
}).
-record(disk_cache_loader_done,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Disk cache loaded ",filecount," files in ",duration, " ms"] :: event_logformat(),
  filecount = undefined :: non_neg_integer(),
  duration = undefined :: milliseconds()
}).


-record(drm_key_fetch_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["error ", error, " ", {description, [description, " "], ""},
  "fetching DRM key ",{resource_id, ["for ",resource_id], ""}, 
  {content_type, [" ",content_type], ""}, " from ", key_url ] :: event_logformat(),
  error = undefined :: any(),
  description = undefined :: binary(),
  resource_id = undefined :: drm_resource_id(),
  key_url = undefined :: url(),
  content_type = undefined :: drm_content_type()
}).

-record(drm_keyserver_is_slow,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["slow ", provider, " drm: keyserver handled url ", url, " for ", duration, " ms"] :: event_logformat()
}).



-record(web_backend_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["web backend ",name, " started"] :: event_logformat(),
  name = undefined :: atom()
}).
-record(web_backend_start_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["web backend ",name, " start error: ",error] :: event_logformat(),
  name = undefined :: atom(),
  error = undefined :: any()
}).
-record(web_backend_is_slow,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["web backend ",name, " handled url ",url, " for ", duration, " ms"] :: event_logformat(),
  name = undefined :: atom(),
  url = undefined :: url(),
  duration = undefined :: seconds()
}).

-record(sql_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["error running query ", 'query', ": ", error, ": ", stacktrace] :: event_logformat()
}).



-record(http_file_cached,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["http_file ", http_file, " is cached to ", cache_file] :: event_logformat(),
  cache_file = undefined :: disk_path(),
  http_file = undefined :: url()
}).

-record(file_reader_delay_starting,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["delaying start of ", count, " simultaneous workers"] :: event_logformat(),
  count = undefined :: non_neg_integer()
}).

%% system monitor events
-record(busy_port,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Process ", orig_pid, " suspended because of busy port ", port] :: event_logformat(),
  group = undefined :: binary(),
  orig_pid = undefined :: pid(),
  port = undefined :: any()
}).

-record(busy_dist_port,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Process ", orig_pid, " suspended because of busy remote port ", port] :: event_logformat(),
  group = undefined :: binary(),
  orig_pid = undefined :: pid(),
  port = undefined :: any()
}).

-record(long_gc,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Long GC of pid ", orig_pid, " takes ", timeout, "ms, ",
                           ", heap size: ", heap_size, "(", old_heap_size, ")",
                           ", heap block: ", heap_block_size, "(", old_heap_block_size, ")",
                           ", stack: ", stack_size,
                           ", mbuf: ", mbuf_size] :: event_logformat(),
  group = undefined :: binary(),
  heap_block_size = undefined :: non_neg_integer(),
  heap_size = undefined :: non_neg_integer(),
  mbuf_size = undefined :: non_neg_integer(),
  old_heap_block_size = undefined :: non_neg_integer(),
  old_heap_size = undefined :: non_neg_integer(),
  orig_pid = undefined :: pid(),
  stack_size = undefined :: non_neg_integer(),
  timeout = undefined :: non_neg_integer()
}).
-record(long_schedule_pid,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Pid ", orig_pid, " runs uninterrupted for ", timeout, "ms, in/out: ", in, "/", out] :: event_logformat(),
  group = undefined :: binary(),
  in = undefined :: any(),
  orig_pid = undefined :: pid(),
  out = undefined :: any(),
  timeout = undefined :: non_neg_integer()
}).
-record(long_schedule_port,{
  ?DEFAULT_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Port ", orig_port, " runs uninterrupted for ", timeout, "ms with op ", port_op] :: event_logformat(),
  group = undefined :: binary(),
  port_op = undefined :: any(),
  orig_port = undefined :: any(),
  timeout = undefined :: non_neg_integer()
}).

-record(memory_usage,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["MEM:",
                             {limit,  [" limit:",  limit],  ""},
                             {total,  [" total:",  total],  ""},
                             {used,   [" used:",   used],   ""},
                             {binary, [" binary:", binary], ""},
                             {top,    [" top:",    top],    ""}] :: event_logformat(),
  binary = undefined :: non_neg_integer(),
  group = undefined :: binary(),
  notify_treshold = undefined :: number(),
  overload_treshold = undefined :: number(),
  top = undefined :: any(),
  total = undefined :: non_neg_integer(),
  usage = undefined :: number(),
  used = undefined :: non_neg_integer()
}).

-record(stream_ad_segment,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "ad: stream segment marked -- stream: ", media,
  {out, [" out_of_network: ", out], ""},
  {eid, [" event_id: ", eid], ""},
  {time, [" at: ", time], ""},
  {duration, [" duration: ", duration], ""},
  {elapsed, [" elapsed: ",  elapsed], ""},
  {remained, [" remained: ",  remained], ""}
] :: event_logformat(),
  out = undefined :: true|false|stalled|continue,
  eid = undefined :: non_neg_integer(),
  duration = undefined :: milliseconds(),
  elapsed = undefined :: ticks(),
  remained = undefined :: ticks(),
  auto_return = undefined :: boolean(),
  pts = undefined :: ticks(),
  ad = undefined :: any()
}).


-record(audio_silence_detected,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Silence detected.", {s, [" At ", s], ""}] :: event_logformat()
}).
-record(audio_silence_end,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Silence ended.", {e, [" At ", e], ""}, {duration, [" Duration: ", duration], ""}] :: event_logformat(),
  duration = undefined :: binary()
}).


-record(chassis_connected,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Streamer connected to chassis"] :: event_logformat(),
  node = undefined :: any()
}).
-record(chassis_disconnected,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Streamer disconnected from chassis"] :: event_logformat(),
  node = undefined :: any()
}).
-record(chassis_coder_turned_off,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Coder ",coder, " was turned off"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_coder_turned_on,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Coder ",coder, " was turned on"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_coder_connected,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Coder ",coder," connected to chassis"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_coder_disconnected,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Coder ",coder," disconnected from chassis"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_starting,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Chassis controller ",full_version," is starting"] :: event_logformat(),
  version = undefined :: server_version(),
  full_version = undefined :: server_version()
}).
-record(chassis_coder_starting,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Chassis is starting controller for coder ",coder] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_dhcp_request,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["DHCP ",coder," request from ether ",ethernet] :: event_logformat(),
  coder = undefined :: non_neg_integer(),
  ethernet = undefined :: binary(),
  boot_file = undefined :: binary(),
  serial_number = undefined :: binary(),
  ip = undefined :: binary()
}).
-record(chassis_tftp_request,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["TFTP ",coder," read request ",path] :: event_logformat(),
  coder = undefined :: non_neg_integer(),
  path = undefined :: disk_path()
}).
-record(chassis_tftp_ready,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["TFTP ",coder," read done ",path, " (",size," bytes) speed ", speed, "Kbps"] :: event_logformat(),
  coder = undefined :: non_neg_integer(),
  path = undefined :: disk_path(),
  size = undefined :: bytes(),
  speed = undefined :: speed()
}).
-record(chassis_nfs_mounted,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Coder ",coder, " mounted its nfs root"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).
-record(chassis_ntp_request,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Coder ",coder," asked whats the time"] :: event_logformat(),
  coder = undefined :: non_neg_integer()
}).


-record(chassis_firmware_remount_rw,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["Remount RW firmware storage"] :: event_logformat(),
  path = undefined :: disk_path()
}).
-record(chassis_firmware_downloaded_manifest,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["Downloaded manifest ",repo,"/",version] :: event_logformat(),
  repo = undefined :: any(),
  version = undefined :: chassis_version()
}).
-record(chassis_firmware_downloaded_file,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["Downloaded file ",path] :: event_logformat(),
  path = undefined :: disk_path()
}).
-record(chassis_firmware_updated_bootloader,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["Updated bootloader to ",repo,"/",version] :: event_logformat(),
  repo = undefined :: any(),
  version = undefined :: chassis_version()
}).
-record(chassis_firmware_remount_ro,{
  ?DEFAULT_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["Remount RO firmware storage"] :: event_logformat(),
  path = undefined :: disk_path()
}).

-record(chassis_reboot,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Chassis is rebooting now"] :: event_logformat()
}).
-record(chassis_shutdown,{
  ?DEFAULT_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Chassis is power off now"] :: event_logformat()
}).

-record(transcoder_benchmark_result,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["The maximum number of threads on the encoder without overload is ", streams] :: event_logformat(),
  streams = undefined :: non_neg_integer()
}).

-record(shutdown_timeout,{
  ?DEFAULT_MEDIA_FIELDS,
  node = undefined :: atom(),
  loglevel = error :: event_loglevel(),
  format = ["Node ", node, " shutdown timeout"] :: event_logformat()
}).
-record(epmd_stalled,{
  ?DEFAULT_MEDIA_FIELDS,
  node = undefined :: atom(),
  record = undefined :: any(),
  loglevel = error :: event_loglevel(),
  format = ["EPMD has stalled record for node ", node, ": ", record] :: event_logformat()
}).
-record(kill_process,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Kill process ", os_name, "(", os_pid, ")"] :: event_logformat(),
  os_name = undefined :: any(),
  os_pid = undefined :: non_neg_integer()
}).


-record(webrtc_open_port,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Opening ", inet, " ", port, "/", transport] :: event_logformat(),
  inet = ipv4 :: network_version(),
  port = undefined :: network_port(),
  transport = undefined :: network_transport(),
  ip = undefined :: network_addr()
}).
-record(webrtc_set_peer,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Set remote peer: ", address, ":", port, "/", transport] :: event_logformat(),
  ip = undefined :: network_addr(),
  port = undefined :: network_port(),
  transport = undefined :: network_transport()
}).


-record(drop_empty_segment,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["Drop empty segment opened_at=", opened_at] :: event_logformat(),
  opened_at = undefined :: seconds()
}).






%  $$$$$$\                                                    
% $$  __$$\                                                   
% $$ /  \__| $$$$$$\  $$\   $$\  $$$$$$\   $$$$$$$\  $$$$$$\  
% \$$$$$$\  $$  __$$\ $$ |  $$ |$$  __$$\ $$  _____|$$  __$$\ 
%  \____$$\ $$ /  $$ |$$ |  $$ |$$ |  \__|$$ /      $$$$$$$$ |
% $$\   $$ |$$ |  $$ |$$ |  $$ |$$ |      $$ |      $$   ____|
% \$$$$$$  |\$$$$$$  |\$$$$$$  |$$ |      \$$$$$$$\ \$$$$$$$\ 
%  \______/  \______/  \______/ \__|       \_______| \_______|









-record(playlist_entry_switch,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Switch to ", entry, " with duration ", duration, " ms"] :: event_logformat(),
  duration = undefined :: ticks(),
  entry = undefined :: stream_name()
}).


-record(epg_changed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["epg table changed for stream: ", media] :: event_logformat(),
  title = undefined :: binary()
}).






-record(mpegts_too_long_adaptation,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["too long (",adaptation_length,") adapdation field on pid ",ts_pid] :: event_logformat(),
  adaptation_length = undefined :: non_neg_integer(),
  ts_pid = undefined :: ts_pid(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).
-record(mpegts_autoselect_program,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["autoselecting program ",program] :: event_logformat(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).
-record(mpegts_cc_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["CC error on pid ", ts_pid, " ", codec, ": exp ", exp_cc, " got ", got_cc, "; ", cc_action] :: event_logformat(),
  cc_action = log :: mpegts_cc_error_action(),
  ts_pid = undefined :: ts_pid(),
  codec = undefined :: frame_codec(),
  exp_cc = undefined :: non_neg_integer(),
  got_cc = undefined :: non_neg_integer(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).

-record(mpegts_reset_decoder,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Reset decoder due to ", reason, " on pid ", ts_pid, " count/limit: ",reset_count,"/",reset_limit] :: event_logformat(),
  ts_pid = undefined :: non_neg_integer(),
  reason = undefined :: binary(),
  reset_count = undefined :: non_neg_integer(),
  reset_limit = undefined :: non_neg_integer(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).
-record(mpegts_flush_frames,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["Flush all frames due to DTS jump: ", old_dts, "->", new_dts, " ",content, " track ", track_id] :: event_logformat(),
  content = undefined :: frame_content(),
  new_dts = undefined :: ticks(),
  old_dts = undefined :: ticks(),
  track_id = undefined :: non_neg_integer(),
  ts_pid = undefined :: non_neg_integer(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).
-record(mpegts_restart_decoder,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["Has to restart decoder because of error limit: ",reset_count] :: event_logformat(),
  ts_pid = undefined :: non_neg_integer(),
  reset_count = undefined :: non_neg_integer(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).
-record(mpegts_decode_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["submedia:",submedia," pid:",ts_pid," class:",class," error:",error," stack:\n",stacktrace] :: event_logformat(),
  ts_pid = undefined :: non_neg_integer(),
  class = undefined :: any(),
  error = undefined :: any(),
  stacktrace = undefined :: any(),
  ip = undefined :: binary(),
  user_id = undefined :: binary()
}).


-record(mpegts_encoder_init_program, {
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["init prog pnr=", program, " vpcr=", vpcr, " from dts=", dts, "; GPCR=", gpcr, " -> ", shift] :: event_logformat(),
  program = undefined :: integer(),
  dts = undefined :: number(),
  gpcr = undefined :: integer(),
  vpcr = undefined :: integer(),
  shift = undefined :: integer()
}).

-record(mpegts_encoder_reset_program, {
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Resetting program ", program, " because of ", reason] :: event_logformat(),
  program = undefined :: integer(),
  reason = undefined :: binary()
}).









-record(dvb_open_error,{
  ?DEFAULT_MEDIA_FIELDS,  
  loglevel = error :: event_loglevel(),
  format = ["Dvb ", media, " open adapter ", adapter, " error: ", error] :: event_logformat(),
  adapter = undefined :: non_neg_integer(),
  error = undefined :: not_found_dvb_card | fatal_open_frontend | open_frontend | fatal_set_property | set_property | start_dvb_dvr
}).



-record(rtsp_resync,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["RTSP ",count," resyncs on '",url,"' after ",bytes," bytes"] :: event_logformat(),
  url = undefined :: url(),
  bytes = undefined :: bytes(),
  count = undefined :: non_neg_integer()
}).
-record(rtsp_desync,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["RTSP full desync on '",url,"' after ",bytes] :: event_logformat(),
  url = undefined :: url(),
  bytes = undefined :: non_neg_integer()
}).
-record(rtsp_broken_data,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["RTSP skip broken data"] :: event_logformat()
}).

-record(h264_rtp_workaround,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["enable ",mode, " trick mode"] :: event_logformat(),
  mode = undefined :: annexb1 | annexb2 | broken_sei
}).





-record(source_multicast,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = undefined :: event_logformat(),
  url = undefined :: url(),
  source = undefined :: hostport()
}).
-record(udp_stats,{
  ?DEFAULT_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["inputs:",inputs, ", bytes:",bytes, ", packets:",packets,", errors:",errors] :: event_logformat(),
  inputs = 0 :: non_neg_integer(),
  bytes = 0 :: bytes(),
  packets = 0 :: non_neg_integer(),
  errors = 0 :: non_neg_integer(),
  now = undefined :: seconds(),
  dummy = undefined :: any()
}).





%  $$$$$$\                                $$\                               
% $$  __$$\                               \__|                              
% $$ /  \__| $$$$$$\   $$$$$$$\  $$$$$$$\ $$\  $$$$$$\  $$$$$$$\   $$$$$$$\ 
% \$$$$$$\  $$  __$$\ $$  _____|$$  _____|$$ |$$  __$$\ $$  __$$\ $$  _____|
%  \____$$\ $$$$$$$$ |\$$$$$$\  \$$$$$$\  $$ |$$ /  $$ |$$ |  $$ |\$$$$$$\  
% $$\   $$ |$$   ____| \____$$\  \____$$\ $$ |$$ |  $$ |$$ |  $$ | \____$$\ 
% \$$$$$$  |\$$$$$$$\ $$$$$$$  |$$$$$$$  |$$ |\$$$$$$  |$$ |  $$ |$$$$$$$  |
%  \______/  \_______|\_______/ \_______/ \__| \______/ \__|  \__|\_______/ 



-define(SESSION_FIELDS, proto = undefined :: protocol(),
  id = undefined :: session_id(),
  source_id = undefined :: session_id(),
  parent_id = undefined :: session_id(),
  ip = undefined :: network_addr(),
  opened_at = undefined :: milliseconds(),
  duration = undefined :: milliseconds(),
  bytes = undefined :: bytes()
).

-define(SESSION_FIELDS2, ?SESSION_FIELDS,
  token = undefined :: session_token(),
  referer = undefined :: url(),
  user_agent = undefined :: user_agent(),
  query_string = undefined :: binary()
).


-define(PLAY_FIELDS, ?SESSION_FIELDS2,
  country = undefined :: country(),
  user_name = undefined :: stream_name(),
  user_id = undefined :: binary()
).





%                                  _ 
%                                 | |
%   ___  _ __   ___ _ __   ___  __| |
%  / _ \| '_ \ / _ \ '_ \ / _ \/ _` |
% | (_) | |_) |  __/ | | |  __/ (_| |
%  \___/| .__/ \___|_| |_|\___|\__,_|
%       | |                          
%       |_|                          
%


-record(source_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["open source ", url, " ",id] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  priority = undefined :: non_neg_integer()
}).

-record(push_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["create push ", url, {retry_limit, [" ",retry_count, "/", retry_limit, " retries"], []}] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  retry_count = undefined :: non_neg_integer(),
  retry_limit = undefined :: non_neg_integer()
}).

-record(play_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["play_opened ",proto] :: event_logformat(),
  ?PLAY_FIELDS
}).


-record(stream_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stream opened"] :: event_logformat(),
  ?SESSION_FIELDS
}).






-record(session_opened,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = [type] :: event_logformat(),
  ?PUBLIC_SESSION_FIELDS,
  access = undefined :: atom(),
  active = undefined :: boolean()
}).







-record(decklink_start_auto,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["decklink capture start device ", device_id, " auto mode"] :: event_logformat(),
  device_id = undefined :: decklink_card_id()
}).
-record(decklink_start_manual,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["decklink capture start device ", device_id, " format:", format_id, " ", format_name, " size:", width, "x", height, "/", field_dominance, " fps:", fps_num, "/", fps_den] :: event_logformat(),
  device_id = undefined :: decklink_card_id(),
  format_id = undefined :: any(),
  format_name = undefined :: any(),
  width = undefined :: pixels(),
  height = undefined :: pixels(),
  field_dominance = undefined :: any(),
  fps_num = undefined :: non_neg_integer(),
  fps_den = undefined :: non_neg_integer()
}).

-record(pusher_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["starting pusher ", url, " (", status, ")"] :: event_logformat(),
  url = undefined :: url(),
  status = undefined :: pusher_status()
}).




%                                  _           _ 
%                                 | |         | |
%   ___ ___  _ __  _ __   ___  ___| |_ ___  __| |
%  / __/ _ \| '_ \| '_ \ / _ \/ __| __/ _ \/ _` |
% | (_| (_) | | | | | | |  __/ (__| ||  __/ (_| |
%  \___\___/|_| |_|_| |_|\___|\___|\__\___|\__,_|
%



-record(source_connected,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["source ", url, " connecting: ", status] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  priority = undefined :: non_neg_integer(),
  status = undefined :: any()
}).

-record(push_connected,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["push ", url, " connecting: ", status] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  status = undefined :: any()
}).

-record(play_connected,{
}).


-record(auth_backend_request,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = 
  ["Backend auth request \"",url,"\" (",identity,",",extra,",",options,"): Code: ",code,", ",
    {auth_time, ["auth_time: ",auth_time,", "], ""},
    {user_id, ["user_id: ",user_id,", "], ""},
    {max_sessions, ["max_sessions: ",max_sessions,", "], ""},
    {location, ["location: ",location,", "], ""},
    "time, ",runtime,"ms"] :: event_logformat(),
  url = undefined :: auth_url(),
  identity = undefined :: session_identity(),
  options = undefined :: any(),
  extra = undefined :: any(),
  code = undefined :: atom(),
  auth_time = undefined :: milliseconds(),
  user_id = undefined :: session_user_id(),
  max_sessions = undefined :: non_neg_integer(),
  location = undefined :: url(),
  runtime = undefined :: milliseconds(),
  ip = undefined :: network_addr()
}).




%      _             _           _ 
%     | |           | |         | |
%  ___| |_ __ _ _ __| |_ ___  __| |
% / __| __/ _` | '__| __/ _ \/ _` |
% \__ \ || (_| | |  | ||  __/ (_| |
% |___/\__\__,_|_|   \__\___|\__,_|
%                                
                                  




-record(source_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["source started ", url, {dts, [" on dts ",dts], ""}, " ", id] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,  
  priority = undefined :: non_neg_integer(),
  dts = undefined :: ticks()
}).

-record(push_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["start push ", proto, " ", url] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2
}).


-record(play_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["play_started ",proto] :: event_logformat(),
  ?PLAY_FIELDS
}).



% deprecated
% Несмотря на то, что этот евент вроде бы стандартный для нашей схемы, мы не знаем, как его заполнять
-record(stream_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stream started"] :: event_logformat()
}).












% deprecated
-record(pusher_started,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["pusher ", url, " started: ", pusher_pid] :: event_logformat(),
  url = undefined :: url(),
  pusher_pid = undefined :: pid()
}).

-record(stream_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["autostart stream"] :: event_logformat()
}).

%                  _       _       
%                 | |     | |      
%  _   _ _ __   __| | __ _| |_ ___ 
% | | | | '_ \ / _` |/ _` | __/ _ \
% | |_| | |_) | (_| | (_| | ||  __/
%  \__,_| .__/ \__,_|\__,_|\__\___|
%       | |                        
%       |_|                        
%


-record(source_updated,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["source is still alive for ",duration," seconds with ",bytes," bytes and ",bitrate,"kbps"] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  priority = undefined :: non_neg_integer(),
  bitrate = undefined :: speed()
}).

-record(push_updated,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["pusher ", url, " sent ",bytes, " bytes"] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2
}).

-record(play_updated,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["play_updated ",proto] :: event_logformat(),
  ?PLAY_FIELDS
}).

-record(stream_updated,{
  ?DEFAULT_MEDIA_FIELDS,
  ?SESSION_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["stream alive"] :: event_logformat(),
  reason = undefined :: atom(),
  sub_id = undefined :: session_id(),
  sub_bytes = undefined :: bytes(),
  sub_opened_at = undefined :: milliseconds(),
  bytes_in = undefined :: bytes(),
  bytes_out = undefined :: bytes()
}).





% deprecated

-record(source_still_alive,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["source is still alive for ",duration," seconds with ",bytes," bytes and ",bitrate,"kbps"] :: event_logformat(),
  url = undefined :: url(),
  connected_at = undefined :: ticks(),
  duration = undefined :: seconds(),
  bytes = undefined :: bytes(),
  id = undefined :: stream_source_id(),
  bitrate = undefined :: speed()
}).




%       _                                 
%      | |                                
%   ___| |__   __ _ _ __   __ _  ___  ___ 
%  / __| '_ \ / _` | '_ \ / _` |/ _ \/ __|
% | (__| | | | (_| | | | | (_| |  __/\__ \
%  \___|_| |_|\__,_|_| |_|\__, |\___||___/
%                          __/ |          
%                         |___/           
%



-record(push_overflowed,{
}).

-record(frames_timed_out,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["frames timeout"] :: event_logformat()
 }).

-record(frames_restored,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["frames restored"] :: event_logformat()
 }).




-record(udp_pusher_reset_timing,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["Resetting UDP pusher timing due to large delay: ", delay, " Queue: ", queue_len, ", Mailbox: ", mailbox_len] :: event_logformat(),
  delay = undefined :: number(),
  queue_len = undefined :: non_neg_integer(),
  mailbox_len = undefined :: non_neg_integer()
}).
-record(udp_pusher_does_not_fit_cbr,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = error :: event_loglevel(),
  format = ["Trimming ", bytes, " bytes on pid ",ts_pid," to fit into CBR. src=",deadline_src] :: event_logformat(),
  ts_pid = undefined :: ts_pid(),
  bytes = undefined :: bytes(),
  deadline_src = undefined :: non_neg_integer()
}).

-record(pusher_ts_queue_exausted,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["pusher TS queue exausted!"] :: event_logformat(),
  url = undefined :: url()
}).

-record(udp_pusher_tune_pid_bitrate,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Tuning pid ", ts_pid, " bitrate after ", b_trimmed, " per ", b_payload, " bytes trimmed: ",
            prev_bitrate, " -> ", bitrate] :: event_logformat(),
  ts_pid = undefined :: ts_pid(),
  b_trimmed = undefined :: bytes(),
  b_payload = undefined :: bytes(),
  prev_bitrate = undefined :: speed(),
  bitrate = undefined :: speed()
}).
-record(udp_pusher_tune_muxer_bitrate,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Tuning muxer bitrate to fit ", media_bitrate, " + ", sys_bitrate, " kbps of data: ",
            prev_bitrate, " -> ", bitrate] :: event_logformat(),
  media_bitrate = undefined :: speed(),
  sys_bitrate = undefined :: speed(),
  prev_bitrate = undefined :: speed(),
  bitrate = undefined :: speed()
}).




%       _                    _ 
%      | |                  | |
%   ___| | ___  ___  ___  __| |
%  / __| |/ _ \/ __|/ _ \/ _` |
% | (__| | (_) \__ \  __/ (_| |
%  \___|_|\___/|___/\___|\__,_|
%                              
 


-record(source_closed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  "source closed ", url,
  {reason, [" reason: ", reason], ""},
  {client_count, [" clients: ", client_count], ""},
  " ", id
  ] :: event_logformat(),
  ?SESSION_FIELDS2,
  priority = undefined :: non_neg_integer(),
  limit = undefined :: non_neg_integer(),
  count = undefined :: non_neg_integer(),
  url = undefined :: url(),
  reason = undefined :: any(),
  bitrate = undefined :: speed(),
  client_count = undefined :: integer()
}).


-record(push_closed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["close push ", proto, " ", url] :: event_logformat(),
  url = undefined :: url(),
  ?SESSION_FIELDS2,
  reason = undefined :: any()
}).

% этот евент нужен, он означает что вообще попытка туда подключиться остановлена
-record(pusher_disabled,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["pusher ", url, " disabled"] :: event_logformat(),
  url = undefined :: url()
}).


-record(play_closed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["play_closed ",proto, " ", {token, [" token:",token],""}, " sid:",id, " reason:",reason] :: event_logformat(),
  ?PLAY_FIELDS,
  closed_at = undefined :: milliseconds(),
  reason = undefined :: any()
}).


-record(stream_closed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["stream closed"] :: event_logformat(),
  ?SESSION_FIELDS,
  bytes_out = undefined :: bytes(),
  reason = undefined :: any()  
}).



% deprecated


-record(session_banned,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["banned session ",identity] :: event_logformat(),
  identity = undefined :: any(),
  code = undefined :: any(),
  reason = undefined :: any()
}).
-record(user_multiroom_denied,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Ban session for user_id: ",user_id, " and token: ",token," due to exceded session limit"] :: event_logformat(),
  ip = undefined :: network_addr(),
  user_id = undefined :: session_user_id(),
  token = undefined :: session_token()
}).


-record(auth_backend_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["failed to run auth backend for ",identity," from ",url,": ",error] :: event_logformat(),
  url = undefined :: auth_url(),
  identity = undefined :: session_identity(),
  extra = undefined :: any(),
  options = undefined :: any(),
  error = undefined :: any()
}).











-record(dvr_mp4_export_start,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Request to export ",from,",",duration," to ",destination] :: event_logformat(),
  from :: non_neg_integer(),
  duration :: non_neg_integer(),
  destination = undefined :: undefined | binary()
}).


-record(dvr_mp4_export_ready,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Exported ",from,",",duration," during ",elapsed_ms,"ms"] :: event_logformat(),
  from :: non_neg_integer(),
  duration :: non_neg_integer(),
  elapsed_ms :: non_neg_integer()
}).


-record(dvr_mp4_export_failed,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Failed to export ",from,",",duration,": ",reason] :: event_logformat(),
  from :: non_neg_integer(),
  duration :: non_neg_integer(),
  reason = undefined :: any()
}).


-record(dvr_mp4_export_aborted,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Export aborted due media change"] :: event_logformat()
}).




-record(pusher_exited,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["pusher ", url, " exited"] :: event_logformat(),
  url = undefined :: url()
}).
-record(pusher_stop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = debug :: event_loglevel(),
  format = ["pusher ", url, " is going to ", type, ": ", reason] :: event_logformat(),
  url = undefined :: url(),
  type = undefined :: shutdown |halt,
  reason = undefined :: any()
}).
-record(pusher_source_lost,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["stopping pusher ", url, " due source lost"] :: event_logformat(),
  url = undefined :: url()
}).
-record(pusher_start_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = warning :: event_loglevel(),
  format = ["pusher ", url, " start error: ", error] :: event_logformat(),
  url = undefined :: url(),
  error = undefined :: any()
}).
-record(push_failure,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["failed to push ", url, ": ",description, {retry_delay, [" will retry in ", retry_delay, "ms"],""}] :: event_logformat(),
  url = undefined :: url(),
  description = undefined :: binary()
}).


% deprecated
-record(publish_forbidden,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  {proto, [proto, " "], ""},
  "publish denied",
  {reason, [" ", reason], ""}
  ] :: event_logformat(),
  ip = undefined :: network_addr(),
  proto = undefined :: publish_protocol()
}).

-record(publish_stopped,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = [
  {proto, [proto, " "], ""},
  "unpublish",
  {record_path, [" record ", record_path], ""},
  {duration, [" duration ", duration], ""},
  {bytes, [" bytes ", bytes], ""}
  ] :: event_logformat(),
  ip = undefined :: network_addr(),
  proto = undefined :: publish_protocol(),
  duration = undefined :: milliseconds(),
  bytes = undefined :: bytes()
}).












-record(transcoder_media_info_error,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["prevent frames from transcoding after gettng media_info without video/audio tracks: ", media_info] :: event_logformat(),
  media_info = undefined :: binary()
}).


-record(transcoder_media_info_recover,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["resume transcoding after receving good media_info: ", media_info] :: event_logformat(),
  media_info = undefined :: binary()
}).

-record(transcoder_frames_drop,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = notice :: event_loglevel(),
  format = ["transcoder dropping ", drops, " frames without media info"] :: event_logformat(),
  drops = undefined :: non_neg_integer()
}).




-record(reboot_on_upgrade,{
  ?DEFAULT_MEDIA_FIELDS,
  loglevel = info :: event_loglevel(),
  format = ["Shutdown node due to restart flussonic (version=",
    launched_version,
    ") with a new version=",
    installed_version] :: event_logformat(),
  launched_version = undefined :: binary(),
  installed_version = undefined :: binary()
}).
